! *********************************************************************
! *** CARBONATE EQUILIBRIA CALCULATIONS MODULE ************************
! *********************************************************************
! sue_co2calc.f90
! Version 0.1
! Andy Ridgwell
! 25/08/99
! *********************************************************************



MODULE sue_co2calc


  USE sue_config
  USE sue_lib
  IMPLICIT NONE
  SAVE


! *** dimension arrays of dissociation constant coefficients for pressure correction,
! *** and assign values as constants [Millero, 1982, 1995] ***
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpH2CO3    = (/ -2.55E+1, +1.27E-1, +0.00E+0, -3.08E+0, +8.77E-2 /)
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpHCO3     = (/ -1.58E+1, -2.19E-2, +0.00E+0, +1.13E+0, -1.47E-1 /)
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpBO3H3    = (/ -2.86E+1, +1.21E-1, -3.21E-4, -3.00E+0, +4.27E-2 /)
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpH2O      = (/ -2.00E+1, +1.12E-1, -1.41E-3, -5.13E+0, +7.94E-2 /)
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpHSO4     = (/ -1.80E+1, +4.66E-2, +3.16E-4, -4.53E+0, +9.00E-2 /)
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpF        = (/ -9.78E+0, -9.00E-3, -9.42E-4, -3.91E+0, +5.40E-2 /)
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpH3PO4    = (/ -1.45E+1, +1.21E-1, -3.21E-4, -2.67E+0, +4.27E-2 /)
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpH2PO4    = (/ -2.31E+1, +1.76E-1, -2.65E-3, -5.15E+0, +9.00E-2 /)
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpHPO4     = (/ -2.66E+0, +2.02E-1, -3.04E-3, -4.08E+0, +7.14E-2 /)
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpCaCO3cal = (/ -4.88E+1, +5.30E-1, +0.00E+0, -1.18E+1, +3.69E-1 /)
  REAL(KIND=fp_1),PARAMETER,DIMENSION(5)::dpCaCO3arg = (/ -4.60E+1, +5.30E-1, +0.00E+0, -1.18E+1, +3.69E-1 /)

! *** list of variable names and descriptions ***
! T =       temperature (Kelvin)
! P =       pressure (bar)
! depth =   depth (in meters)
! S =       salinity (parts per thousand)
! I =       ionic strength of seawater
! tot_alk = total alkalinity
! sum_CO2 = total dissolved inorganic carbon
! tot_B =   total boric acid
! tot_PO4 = total dissolved phosphate
! tot_Si =  total dissolved silicate
! k =       ration of the 1st and 2nd carbonic acid dissociation constants
! k1 =      1st carbonic acid dissociation constant
! k2 =      2nd carbonic acid dissociation constant
! kB =      Boric acid dissociation constant
! kW =      dissociation constant of water
! kF =      dissociation constant of HF
! kHSO4 =   dissociation constant of H2SO4
! kP1 =     1st dissociation constant of phosphoric acid
! kP2 =     2nd dissociation constant of phosphoric acid
! kP3 =     3rd dissociation constant of phosphoric acid
! kSi =     dissociation constant of silicic acid
! kcal =    saturation constant for calcite
! karg =    saturation constant for aragonite
! Q_CO2 =   Henry's Law constant for CO2
! Q_O2 =    Henry's Law constant for O2
! c_CO2 =   [CO2(aq)]
! c_CO3 =   [CO3--]
! c_HCO3 =  [HCO3-]
! f_CO2 =   fugacity of cO2
! c_F =     [F-]
! c_SO4 =   [SO4--]
! c_Ca =    [Ca++]
! ohmcal =  saturation state of calcium carbonate in seawater (for calcite)
! ohmarg =  saturation state of calcium carbonate in seawater (for aragonite)


CONTAINS
  
  
  
! *******************************************************************
! SEA WATER CARBONATE EQUILIBRIA CHEMISTRY CALCULATION SUBROUTINE
! To estimate CO2 fugacity and CaCO3 saturation state given the following:
! total alkalinity
! total dissolved inorganic carbon
! salinity
! temperature
! pressure / water depth
! phosphate concentration
! silicate concentration
! *******************************************************************
  
  SUBROUTINE carbcalc_im(T,S,depth,tot_alk,sum_CO2,tot_PO4,tot_Si,f_CO2,ohm_cal,ohm_arg)
    IMPLICIT NONE
    
!   *** dummy argument declarations ***
    REAL(KIND=fp_1),INTENT(IN)::T,S,depth,tot_alk,sum_CO2,tot_PO4,tot_Si
    REAL(KIND=fp_1),INTENT(OUT)::f_CO2,ohm_cal,ohm_arg
!   *** declare local variables ***
    REAL(KIND=fp_1)::P
    REAL(KIND=fp_1)::k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,zed
    REAL(KIND=fp_1)::tot_B,CO2_alk,c_CO2,c_CO3,c_HCO3,c_F,c_SO4,c_Ca
    REAL(KIND=fp_1)::c_H,c_H_old,c_H1,c_H2
    
!   *** Convert from depth (in meters) to an approximate pressure (in bar) ***
!   NOTE: 1 meter in depth is approximately equivalent to a pressure of 1 dbar.
!   The error in this approximation is only about 3% at a depth of 10000 m.
    P = depth/10.0
    
!   *** CALCULATE REQUIRED SEA WATER CONSTANTS ***
    CALL co2const(S,T,P,k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,tot_B,c_Ca)
    
!   guess an initial value for [H+]SWS
    c_H = 7E-9
    
!   *** start of main loop ***
    DO
     
!     Store previous value of [H+]SWS
      c_H_old=c_H
    
!     Calculate carbonate alkalinity.
!     This uses a simple approximation for the contribution of the dissociation of phosphoric acid to carbonate alkalinity, which introduces an error in CO2_alk of no more than 0.02% under thypical oceanic conditions.
      CO2_alk = tot_alk - (kW/c_H - c_H) &
                        - ((kB*tot_B)/(c_H+kB)) &
                        - ((kSi*tot_Si)/(c_H+kSi)) &
                        - 1.1*tot_PO4
    
!     Calculation of carbonate parameters (see Skirrow [1975] for details)
      zed =     ((4*CO2_alk + sum_CO2*k - CO2_alk*k)**2 + 4*(k - 4)*CO2_alk**2)**0.5
      c_HCO3 =  (sum_CO2*k - zed)/(k - 4)
      c_CO3 =   (CO2_alk*k - sum_CO2*k - 4*CO2_alk + zed)/(2*(k - 4))
      c_CO2 =   sum_CO2 - CO2_alk + (CO2_alk*k - sum_CO2*k - 4*CO2_alk + zed)/(2*(k - 4))
    
!     Calculate CO2 fugacity from [CO2(aq)]
      f_CO2 = c_CO2/Q_CO2

!     Calculate the saturation state of CaCO3 (wrt both calcite and aragonite)
      ohm_cal = c_Ca*c_CO3/kcal
      ohm_arg = c_Ca*c_CO3/karg
            
!     *** Estimate new [H+]SWS ***
!     Make two different estimates of pH using the two dissociation constants of carbonic acid
!     and use these to make a more accurate estimate of [H+]SWS.
      c_H1 =  k1*c_CO2/c_HCO3
      c_H2 =  k2*c_HCO3/c_CO3
      c_H =   SQRT(c_H1*c_H2)

!     Check for numerical instability
      IF ((c_H1 < 0.0) .OR. (c_H2 < 0.0)) THEN
	PRINT*,""
	PRINT*,"FATAL ERROR [sue_co2calc.f90, calccarb_im]:"
	PRINT*,"numerical instability"
	PRINT*,""
	STOP
      ENDIF
    
      ! *** End of loop test: % change < 0.1 between successive estimations of [H+] ***
      IF (ABS(1 - c_H/c_H_old) < 0.001) EXIT
    
    END DO
    
  END SUBROUTINE carbcalc_im
! *******************************************************************

  

! *******************************************************************
! Procedure to calculate carbonate equilibrium, returning [Ca], [CO3] CaCO3 saturation state,
! taking values of dissociation constants explicitly as an input (in matrix form)
! also returning pH (NBS scale)
! *******************************************************************
  SUBROUTINE calccarb_ex(const,S,tot_alk,sum_CO2,tot_PO4,tot_Si,conc_Ca, &
    & fug_CO2,conc_CO2,conc_CO3,conc_HCO3,ohm_cal,ohm_arg,dCO3_cal,dCO3_arg,pH)
    IMPLICIT NONE
! dummy variables
    REAL(KIND=fp_1),INTENT(in),DIMENSION(10)::const
    REAL(KIND=fp_1),INTENT(IN)::S,tot_alk,sum_CO2,tot_PO4,tot_Si
    REAL(KIND=fp_1),INTENT(OUT)::conc_Ca,fug_CO2,conc_CO2,conc_CO3,conc_HCO3
    REAL(KIND=fp_1),INTENT(OUT)::ohm_cal,ohm_arg,dCO3_cal,dCO3_arg,pH
! local variables
    REAL(KIND=fp_1)::tot_B,tot_Ca
    REAL(KIND=fp_1)::k,k1,k2,kB,kW,kSi,kcal,karg,zed,Q_CO2
    REAL(KIND=fp_1)::CO2_alk,c_CO2,c_CO3,c_HCO3
    REAL(KIND=fp_1)::c_H,c_H_old,c_H1,c_H2
    REAL(KIND=fp_1)::sat_cal,sat_arg
! estimate borate and Ca concentrations (from salinity) and pressure (from depth in m)
    tot_B  = calc_B(S)
    tot_Ca = calc_Ca(S)
! copy imported constants
    k     = const(1)
    k1    = const(2)
    k2    = const(3)
    kB    = const(4)
    kW    = const(5)
    kSi   = const(6)
    kcal  = const(7)
    karg  = const(8)
    Q_CO2 = const(9)
! calculate value of [CO3--] at calcite and aragonite saturation
! NOTE: this assumes that the value of omega is unity at saturation (definition!)
    sat_cal = kcal * 1.0 / tot_Ca
    sat_arg = karg * 1.0 / tot_Ca
! calculation loop
    c_H = 7E-9    
    DO     
      c_H_old=c_H   
      CO2_alk = tot_alk - (kW/c_H - c_H) &
                        - ((kB*tot_B)/(c_H+kB)) &
                        - ((kSi*tot_Si)/(c_H+kSi)) &
                        - 1.1*tot_PO4   
      zed =     ((4*CO2_alk + sum_CO2*k - CO2_alk*k)**2 + 4*(k - 4)*CO2_alk**2)**0.5
      c_HCO3 =  (sum_CO2*k - zed)/(k - 4)
      c_CO3 =   (CO2_alk*k - sum_CO2*k - 4*CO2_alk + zed)/(2*(k - 4))
      c_CO2 =   sum_CO2 - CO2_alk + (CO2_alk*k - sum_CO2*k - 4*CO2_alk + zed)/(2*(k - 4))        
      c_H1 =  k1*c_CO2/c_HCO3
      c_H2 =  k2*c_HCO3/c_CO3
      IF ((c_H1 < 0.0) .OR. (c_H2 < 0.0)) THEN
	PRINT*,""
	PRINT*,"FATAL ERROR [sue_co2calc.f90, calccarb_ex]:"
	PRINT*,"numerical instability"
	PRINT*,""
	STOP
      ENDIF
      c_H =   SQRT(c_H1*c_H2)    
      IF (ABS(1 - c_H/c_H_old) < 0.001) EXIT   
    END DO
! calculate result variables
    conc_Ca   = tot_Ca
    conc_CO2  = c_CO2
    conc_CO3  = c_CO3
    conc_HCO3 = c_HCO3
    fug_CO2   = conc_CO2 / Q_CO2 
    ohm_cal   = tot_Ca * c_CO3 / kcal
    ohm_arg   = tot_Ca * c_CO3 / karg  
    pH        = -LOG10(c_H)
    dCO3_cal = c_CO3 - sat_cal
    dCO3_arg = c_CO3 - sat_arg
  END SUBROUTINE calccarb_ex
! *******************************************************************



! *******************************************************************
! Procedure to calculate carbonate equilibrium, returning c_H, c_HCO3, c_CO3, c_CO2 and f_CO2,
! taking values of dissociation constants explicitly as an input (in matrix form)
! For surface carbonate system equilibrium calculations
! *******************************************************************
  SUBROUTINE calccarb_sur(c,const,S,tot_alk,sum_CO2,tot_PO4,tot_Si,f_CO2,c_CO2,c_CO3,c_HCO3,c_H)
    IMPLICIT NONE
! dummy variables
    INTEGER,INTENT(IN)::c
    REAL(KIND=fp_1),INTENT(in),DIMENSION(10)::const
    REAL(KIND=fp_1),INTENT(IN)::S,tot_alk,sum_CO2,tot_PO4,tot_Si
    REAL(KIND=fp_1),INTENT(INout)::f_CO2,c_CO2,c_CO3,c_HCO3,c_H
! local variables
    REAL(KIND=fp_1)::P
    REAL(KIND=fp_1)::tot_B
    REAL(KIND=fp_1)::k,k1,k2,kB,kW,kSi,Q_CO2,zed
    REAL(KIND=fp_1)::CO2_alk
    REAL(KIND=fp_1)::c_H_old,c_H1,c_H2
! estimate borate concentration (from salinity) and pressure (from depth in m)
    tot_B = calc_B(S)
! copy imported constants
    k     = const(1)
    k1    = const(2)
    k2    = const(3)
    kB    = const(4)
    kW    = const(5)
    kSi   = const(6)
    Q_CO2 = const(9) 
! calculation loop
!!$    c_H = 7E-9 
    DO     
      c_H_old = c_H   
      CO2_alk = tot_alk - (kW / c_H - c_H) &
                        - ((kB * tot_B) / (c_H + kB)) &
                        - ((kSi * tot_Si) / (c_H + kSi)) &
                        - 1.1 * tot_PO4   
      zed    =  ((4.0 * CO2_alk + sum_CO2 * k - CO2_alk * k)**2 + 4.0* (k - 4.0) * CO2_alk**2)**0.5
      c_HCO3 =  (sum_CO2*k - zed) / (k - 4.0)
      c_CO3  =  (CO2_alk * k - sum_CO2 * k - 4.0 * CO2_alk + zed) / (2.0 * (k - 4.0))
      c_CO2  =  sum_CO2 - CO2_alk + (CO2_alk * k - sum_CO2 * k - 4.0 * CO2_alk + zed) / (2.0 * (k - 4.0))     
      c_H1   =  k1 * c_CO2 / c_HCO3
      c_H2   =  k2 * c_HCO3 / c_CO3
      IF ((c_H1 < 0.0) .OR. (c_H2 < 0.0)) THEN
	PRINT*,""
	PRINT*,"FATAL ERROR [sue_co2calc.f90, calccarb_sur]:"
	PRINT*,"numerical instability"
	PRINT*,c
	PRINT*,tot_alk,sum_CO2
	PRINT*,c_HCO3,c_CO3,c_CO2
	PRINT*,""
	STOP
      ENDIF
      c_H =   SQRT(c_H1 * c_H2)
      IF (ABS(1.0 - c_H / c_H_old) < 0.001) EXIT   
    END DO
    f_CO2 = c_CO2 / Q_CO2 
  END SUBROUTINE calccarb_sur
! *******************************************************************



! *******************************************************************
! Function to calculate carbonate equilibrium, returning CO2 fugacity only, taking values of dissociation constants explicitly as an input (in matrix form)
! *******************************************************************
  FUNCTION calcfug_ex(const,S,tot_alk,sum_CO2,tot_PO4,tot_Si)
  IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calcfug_ex
! dummy variables
    REAL(KIND=fp_1),INTENT(in),DIMENSION(10)::const
    REAL(KIND=fp_1),INTENT(IN)::S,tot_alk,sum_CO2,tot_PO4,tot_Si
! local variables
    REAL(KIND=fp_1)::P
    REAL(KIND=fp_1)::tot_B
    REAL(KIND=fp_1)::k,k1,k2,kB,kW,kSi,Q_CO2,zed
    REAL(KIND=fp_1)::CO2_alk,c_CO2,c_CO3,c_HCO3
    REAL(KIND=fp_1)::c_H,c_H_old,c_H1,c_H2
! estimate borate concentration (from salinity) and pressure (from depth in m)
    tot_B = calc_B(S)
! copy imported constants
    k     = const(1)
    k1    = const(2)
    k2    = const(3)
    kB    = const(4)
    kW    = const(5)
    kSi   = const(6)
    Q_CO2 = const(9) 
! calculation loop
    c_H = 7E-9    
    DO     
      c_H_old = c_H   
      CO2_alk = tot_alk - (kW/c_H - c_H) &
                        - ((kB*tot_B)/(c_H+kB)) &
                        - ((kSi*tot_Si)/(c_H+kSi)) &
                        - 1.1*tot_PO4   
      zed =     ((4*CO2_alk + sum_CO2*k - CO2_alk*k)**2 + 4*(k - 4)*CO2_alk**2)**0.5
      c_HCO3 =  (sum_CO2*k - zed)/(k - 4)
      c_CO3 =   (CO2_alk*k - sum_CO2*k - 4*CO2_alk + zed)/(2*(k - 4))
      c_CO2 =   sum_CO2 - CO2_alk + (CO2_alk*k - sum_CO2*k - 4*CO2_alk + zed)/(2*(k - 4))     
      c_H1 =  k1*c_CO2/c_HCO3
      c_H2 =  k2*c_HCO3/c_CO3
      IF ((c_H1 < 0.0) .OR. (c_H2 < 0.0)) THEN
	PRINT*,""
	PRINT*,"FATAL ERROR [sue_co2calc.f90, calcfug_ex]:"
	PRINT*,"numerical instability"
	PRINT*,""
	STOP
      ENDIF
      c_H =   SQRT(c_H1*c_H2)    
      IF (ABS(1 - c_H/c_H_old) < 0.001) EXIT   
    END DO
! return result
    calcfug_ex = c_CO2/Q_CO2 
  END FUNCTION calcfug_ex
! *******************************************************************


  
! *******************************************************************
! Procedure to calculate required equilibrium constants
! and estimate relevant elemental concentrations in sea water.
! *******************************************************************
  
  SUBROUTINE co2const(S,T,P,k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,tot_B,c_Ca)
    IMPLICIT NONE
  
!   *** Dummy argument declarations ***
    REAL(KIND=fp_1),INTENT(IN)::S,T,P
    REAL(KIND=fp_1),INTENT(INOUT)::k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,tot_B,c_Ca
!   *** Declare local variables ***
    REAL(KIND=fp_1)::scale_pH,scale_conc,scale_P,S_p15,S_p05,T_p20,T_ln,T_log,I,I_p05,Tc,RT
    
!   *** List of local variable names and descriptions ***
!   scale_pH =    ln of pH scale correction factor
!   scale_conc =  ln of concentration scale conversion factor (molarity to mol/(kg soln))
!   scale_P =     ln of pressure correction factor
!   S_p15 =       S^1.5
!   S_p05 =       S^0.5
!   T_ln =        ln(T)
!   T_log =       log10(T)
!   I_p05 =       I^0.5
!   Tc =          T (degrees centigrade)
!   RT =          const_R * T
    
!   *** Calculate local salinity- and temperature-derived constants ***
    S_p15 = S**1.5
    S_p05 = S**0.5
    T_ln =  LOG(T)
    T_log = LOG10(T)
    Tc =    T-const_zeroC
    RT  =   const_R*T
    
!   *** Estimate concentrations of various chemical species from salinity ***
    tot_B = calc_B(S)
    c_Ca  = calc_Ca(S)
    
!   *** Calculate the ionic strength of seawater and associated derived local constant ***
    I = calc_I(S)
    I_p05 = I**0.5
    
! *** Calculate pH conversion factor ***
! Because the pH correction factor in only small, and has almost no dependence on salinity and is approximately linear wrt temperature, the full correction factor to convert between the different pH scales has been replaced with a simple linear approximation, adding an error in kB of no more than about 0.03% under all conditions of interest.
    scale_pH = -5.231E-2 + 2.453E-4*T
    
!   *** calculate concentration units conversion factor [Millero, 1982] ***
    scale_conc = LOG(1 - 0.001055*S)
    
!   *** Calculate dissociation constants ***
    SELECT CASE (miscopt(mopt_co2calc_alt))
    CASE ("1")
      k1 = EXP(calc_lnk1_Millero(T,T_ln,S,S_p05,S_p15) + corr_p(Tc,P,RT,dpH2CO3))
      k2 = EXP(calc_lnk2_Millero(T,T_ln,S,S_p05,S_p15) + corr_p(Tc,P,RT,dpHCO3))
    CASE ("2")
      k1 = EXP(calc_lnk1_Mehrbach_DicksonMillero(T,T_ln,S) + corr_p(Tc,P,RT,dpH2CO3))
      k2 = EXP(calc_lnk2_Mehrbach_DicksonMillero(T,T_ln,S) + corr_p(Tc,P,RT,dpHCO3))
    END SELECT
!   Use the Dickson [1990] data fitted for the mol/kg-SW scale
    kB = EXP(calc_lnkB(T,T_ln,S,S_p05,S_p15) + scale_pH + corr_p(Tc,P,RT,dpBO3H3))
!   Correction to KH20 as the values given in Millero [1995] are for fresh water, so use values from Millero [1982] for the seawater system instead
    kW = EXP(calc_lnkW(T,T_ln,S,S_p05) + corr_p(Tc,P,RT,dpH2O))
!   No specific pressure correction for Si was available but Millero [1995] estimated the correction factors from those for boric acid.
    kSi = EXP(calc_lnkSi(T,T_ln,I,I_p05) + scale_conc + corr_p(Tc,P,RT,dpBO3H3))
    
!   *** Calculate ratio of K1/K2 ***
    k = k1/k2
                                                                                
!   *** Calculate solubility coefficients of CaCO3: calcite and aragonite.
    kcal = EXP(corr_p(Tc,P,RT,dpCaCO3cal)) * 10**(calc_logkcal(T,T_log,S,S_p05,S_p15))
    karg = EXP(corr_p(Tc,P,RT,dpCaCO3arg)) * 10**(calc_logkarg(T,T_log,S,S_p05,S_p15))
    
!   *** Calculate Q_CO2 ***
    Q_CO2 = EXP(calc_lnQ_CO2(T,S))
  
  END SUBROUTINE co2const
! *******************************************************************
  

  
! *******************************************************************
! Procedure to calculate required equilibrium constants and estimate relevant elemental concentrations in sea water, returning them in matrix form.
! *******************************************************************
  SUBROUTINE co2const_ex(T,S,D,const)
    IMPLICIT NONE
!   * dummy variables
    REAL(KIND=fp_1),INTENT(IN)::T,S,D
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(10)::const
!   * local variables
    REAL(KIND=fp_1)::scale_pH,scale_conc,scale_P,S_p15,S_p05,T_p20,T_ln,T_log,I,I_p05,Tc,RT,P    
!   *
    S_p15      = S**1.5
    S_p05      = S**0.5
    T_ln       = LOG(T)
    T_log      = LOG10(T)
    Tc         = T-const_zeroC
    RT         = const_R*T
    P          = D/10.0
    I          = calc_I(S)
    I_p05      = I**0.5   
    scale_pH   = -5.231E-2 + 2.453E-4*T
    scale_conc = LOG(1 - 0.001055*S) 
    SELECT CASE (miscopt(mopt_co2calc_alt))
    CASE ("1")
      const(2) = EXP(calc_lnk1_Millero(T,T_ln,S,S_p05,S_p15) + corr_p(Tc,P,RT,dpH2CO3))
      const(3) = EXP(calc_lnk2_Millero(T,T_ln,S,S_p05,S_p15) + corr_p(Tc,P,RT,dpHCO3))
    CASE ("2")
      const(2) = EXP(calc_lnk1_Mehrbach_DicksonMillero(T,T_ln,S) + corr_p(Tc,P,RT,dpH2CO3))
      const(3) = EXP(calc_lnk2_Mehrbach_DicksonMillero(T,T_ln,S) + corr_p(Tc,P,RT,dpHCO3))
    END SELECT
    const(4)   = EXP(calc_lnkB(T,T_ln,S,S_p05,S_p15) + scale_pH + corr_p(Tc,P,RT,dpBO3H3))
    const(5)   = EXP(calc_lnkW(T,T_ln,S,S_p05) + corr_p(Tc,P,RT,dpH2O))
    const(6)   = EXP(calc_lnkSi(T,T_ln,I,I_p05) + scale_conc + corr_p(Tc,P,RT,dpBO3H3))
    const(1)   = const(2)/const(3)
    const(7)   = EXP(corr_p(Tc,P,RT,dpCaCO3cal)) * 10**(calc_logkcal(T,T_log,S,S_p05,S_p15))
    const(8)   = EXP(corr_p(Tc,P,RT,dpCaCO3arg)) * 10**(calc_logkarg(T,T_log,S,S_p05,S_p15))
    const(9)   = EXP(calc_lnQ_CO2(T,S))
    const(10)  = EXP(calc_lnQ_O2(T,S))
  END SUBROUTINE co2const_ex
! *******************************************************************



! *******************************************************************
! ***                                                            ****
! *******************************************************************
  FUNCTION calcfug(T,S,depth,tot_alk,sum_CO2,tot_PO4,tot_Si)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calcfug
! dummy argument declarations
    REAL(KIND=fp_1),INTENT(IN)::T,S,depth,tot_alk,sum_CO2,tot_PO4,tot_Si
! declare local variables
    REAL(KIND=fp_1)::P
    REAL(KIND=fp_1)::k,k1,k2,kB,kW,kP1,kP2,kP3,kSi,kcal,karg,Q_CO2,zed
    REAL(KIND=fp_1)::tot_B,CO2_alk,c_CO2,c_CO3,c_HCO3,c_Ca
    REAL(KIND=fp_1)::c_H,c_H_old,c_H1,c_H2
    REAL(KIND=fp_1)::fug
    
!   *** Convert from depth (in meters) to an approximate pressure (in bar) ***
!   NOTE: 1 meter in depth is approximately equivalent to a pressure of 1 dbar.
!   The error in this approximation is only about 3% at a depth of 10000 m.
    P = depth/10.0
    
!   *** CALCULATE REQUIRED SEA WATER CONSTANTS ***
    CALL co2const(S,T,P,k,k1,k2,kB,kW,kSi,kcal,karg,Q_CO2,tot_B,c_Ca)
    
!   guess an initial value for [H+]SWS
    c_H = 7E-9
    
!   *** start of main loop ***
    DO
     
!     Store previous value of [H+]SWS
      c_H_old=c_H
    
!     Calculate carbonate alkalinity.
      CO2_alk = tot_alk - (kW/c_H - c_H) &
                        - ((kB*tot_B)/(c_H+kB)) &
                        - ((kSi*tot_Si)/(c_H+kSi)) &
                        - 1.1*tot_PO4
    
!    Calculation of carbonate parameters (see Skirrow [1975] for details)
      zed =     ((4*CO2_alk + sum_CO2*k - CO2_alk*k)**2 + 4*(k - 4)*CO2_alk**2)**0.5
      c_HCO3 =  (sum_CO2*k - zed)/(k - 4)
      c_CO3 =   (CO2_alk*k - sum_CO2*k - 4*CO2_alk + zed)/(2*(k - 4))
      c_CO2 =   sum_CO2 - CO2_alk + (CO2_alk*k - sum_CO2*k - 4*CO2_alk + zed)/(2*(k - 4))
    
!     Convert [CO2(aq)] to fugacity
      fug = c_CO2/Q_CO2
            
!     *** Estimate new [H+]SWS ***
!     Make two different estimates of pH using the two dissociation constants of carbonic acid
!     and use these to make a more accurate estimate of [H+]SWS.
      c_H1 =  k1*c_CO2/c_HCO3
      c_H2 =  k2*c_HCO3/c_CO3
      c_H =   SQRT(c_H1*c_H2)
    
!     *** End of loop test: % change < 0.1 between successive estimations of [H+] ***
      IF (ABS(1 - c_H/c_H_old) < 0.001) EXIT
    
    END DO

  calcfug = fug

  END FUNCTION calcfug
! *******************************************************************


  
! ************************************************
! *** Calculate the ionic strength of seawater ***
! ************************************************
  FUNCTION calc_I(S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_I
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::S
! the ionic strength is calculated after Millero [1982]
    calc_I = 19.92*S/(1000 - 1.005*S)
  END FUNCTION calc_I
! ************************************************
  

! ******************************************
! *** Estimate total boron concentration ***
! ******************************************
  FUNCTION calc_B(S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_B
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::S
! Total boroic acid in seawater from Millero [1982, 1995].
! This differs from the relation used in the "CO2SYS" model by Lewis and Wallace,
! which was from Uppstrom [1974].
    calc_B=(1.189E-5)*S
  END FUNCTION calc_B
! ******************************************
  

! ************************************
! *** Estimate concentration of F- ***
! ************************************
  FUNCTION calc_F(S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_F
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::S
! total F- concentration (concentration scale = ???)
! [Millero, 1982]
    calc_F = (2E-6)*S
  END FUNCTION calc_F
! ************************************
  

! ***************************************
! *** Estimate concentration of SO4-- ***
! ***************************************
  FUNCTION calc_SO4(S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_SO4
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::S
! total SO4-- concentration (concentration scale = ???)
! [Millero, 1982]
    calc_SO4 = (8.37E-4)*S
  END FUNCTION calc_SO4
! ***************************************


! **************************************
! *** Estimate concentration of Ca++ ***
! **************************************
  FUNCTION calc_Ca(S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_Ca
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::S
! Total Ca++ concentration [Millero, 1982].
    calc_Ca = (2.937E-4)*S
  END FUNCTION calc_Ca
! **************************************
    

! **********************************************
! *** Calculate ln(K1) dissociation constant ***
! **********************************************
  FUNCTION calc_lnk1_Millero(T,T_ln,S,S_p05,S_p15)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnk1_Millero
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S,S_p05,S_p15
! first carbonic acid apparent ionization constant
! [Millero, 1995]
    calc_lnk1_Millero = 2.18867 - 2275.0360/T - 1.468591 * T_ln &
                + (-0.138681 - 9.33291/T)*S_p05 + 0.0726483*S - 0.00574938*S_p15
  END FUNCTION calc_lnk1_Millero
! ******************************************
  

! **********************************************
! *** Calculate ln(K2) dissociation constant ***
! **********************************************
  FUNCTION calc_lnk2_Millero(T,T_ln,S,S_p05,S_p15)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnk2_Millero
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S,S_p05,S_p15
! second carbonic acid apparent ionization constant
! [Millero, 1995]
    calc_lnk2_Millero = -0.84226 - 3741.1288/T - 1.437139*T_ln &
                + (-0.128417 - 24.41239/T)*S_p05 + 0.1195308*S - 0.00912840*S_p15
  END FUNCTION calc_lnk2_Millero
! ******************************************
    

! **********************************************
! *** Calculate ln(K1) dissociation constant ***
! **********************************************
  FUNCTION calc_lnk1_Mehrbach(T,T_ln,S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnk1_Mehrbach
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S
! first carbonic acid apparent ionization constant
! [Mehrbach et al., 1973]
! NOTE: NBS pH scale
    calc_lnk1_Mehrbach = 0.0
  END FUNCTION calc_lnk1_Mehrbach
! ******************************************
  
  
! **********************************************
! *** Calculate ln(K2) dissociation constant ***
! **********************************************
  FUNCTION calc_lnk2_Mehrbach(T,T_ln,S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnk2_Mehrbach
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S
! second carbonic acid apparent ionization constant
! [Mehrbach et al., 1973]
! NOTE: NBS pH scale
    calc_lnk2_Mehrbach = 0.0
  END FUNCTION calc_lnk2_Mehrbach
! ******************************************
    

! **********************************************
! *** Calculate ln(K1) dissociation constant ***
! **********************************************
  FUNCTION calc_lnk1_Mehrbach_DicksonMillero(T,T_ln,S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnk1_Mehrbach_DicksonMillero
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S
! first carbonic acid apparent ionization constant
! [Dickson and Millero, 1987] (refit of [Mehrbach et al., 1973])
! NOTE: SWS pH scale
    calc_lnk1_Mehrbach_DicksonMillero = LOG &
      & ( &
      &   10**( &
      &    -( &
      &       (3670.7 / T) - 62.008 + 9.7944 * T_ln &
      &       - 0.0118 * S + 0.000116 * S**2 &
      &     ) &
      &   ) &
      & )
  END FUNCTION calc_lnk1_Mehrbach_DicksonMillero
! ******************************************
  
  
! **********************************************
! *** Calculate ln(K2) dissociation constant ***
! **********************************************
  FUNCTION calc_lnk2_Mehrbach_DicksonMillero(T,T_ln,S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnk2_Mehrbach_DicksonMillero
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S
! second carbonic acid apparent ionization constant
! [Dickson and Millero, 1987] (refit of [Mehrbach et al., 1973])
! NOTE: SWS pH scale
    calc_lnk2_Mehrbach_DicksonMillero = LOG &
      & ( &
      &   10**( &
      &    -( &
      &       (1394.7 / T) + 4.777 &
      &       - 0.0184 * S + 0.000118 * S**2 &
      &     ) &
      &   ) &
      & )
  END FUNCTION calc_lnk2_Mehrbach_DicksonMillero
! ******************************************
  

! ************************
! *** Calculate ln(KB) ***
! ************************
  FUNCTION calc_lnkB(T,T_ln,S,S_p05,S_p15)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnkB
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S,S_p05,S_p15
! apparent ionization constant of boric acid
! with concentrations as moles per kg of seawater
! [Dickson, 1990]
    calc_lnkB = (148.0248 + 137.1942*S_p05 + 1.62142*S) &
                + (-8966.90 - 2890.53*S_p05 - 77.942*S + 1.728*S_p15 - 0.0996*S*S)/T &
                + (-24.4344 - 25.085*S_p05 - 0.2474*S)*T_ln &
                + (0.053105*S_p05)*T
  END FUNCTION calc_lnkB
! ************************
  

! ************************
! *** Calculate ln(KW) ***
! ************************
  FUNCTION calc_lnkW(T,T_ln,S,S_p05)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnkW
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S,S_p05
! apparent ionization constant of water [Millero, 1992]
    calc_lnkW = 148.9802 - 13847.26/T - 23.6521*T_ln &
                + (-5.977 + 118.67/T + 1.0495*T_ln)*S_p05 - 0.01615*S
  END FUNCTION calc_lnkW
! ************************
  

! ************************
! *** Calculate ln(KF) ***
! ************************
  FUNCTION calc_lnkF(T,I_p05)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnkF
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,I_p05
! calculation of the dissociation constant of HF
    calc_lnkF = -1590.2/T + 12.6421 - 1.525*I_p05
  END FUNCTION calc_lnkF
! ************************
  

! ****************************
! *** Calculate ln(KP1) ***
! ****************************
  FUNCTION calc_lnkP1(T,T_ln,S,S_p05)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnkP1
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S,S_p05
! calculation of the 1st dissociation constant of phosphoric acid in seawater
    calc_lnkP1 =  115.54 - 4576.752/T - 18.453*T_ln &
                  + (0.69171 - 106.736/T)*S_p05 + (-0.01844 - 0.65643/T)*S
  END FUNCTION calc_lnkP1
! ****************************
  

! ****************************
! *** Calculate ln(KP2) ***
! ****************************
  FUNCTION calc_lnkP2(T,T_ln,S,S_p05)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnkP2
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S,S_p05
! calculation of the 2nd dissociation constant of phosphoric acid in seawater
    calc_lnkP2 =  172.1033 - 8814.715/T - 27.927*T_ln &
                  + (1.3566 - 160.340/T)*S_p05 + (-0.05778 + 0.37335/T)*S
  END FUNCTION calc_lnkP2
! ****************************
  

! ****************************
! *** Calculate ln(KP3) ***
! ****************************
  FUNCTION calc_lnkP3(T,T_ln,S,S_p05)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnkP3
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,S,S_p05
! calculation of the 3rd dissociation constant of phosphoric acid in seawater
    calc_lnkP3 =  -18.126 - 3070.75/T &
                  + (2.81197 + 17.27039/T)*S_p05 + (-0.09984 - 44.99486/T)*S
  END FUNCTION calc_lnkP3
! ****************************
  

! ****************************
! *** Calculate ln(KH2SO4) ***
! ****************************
  FUNCTION calc_lnkHSO4(T,I_p05)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnkHSO4
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,I_p05
! calculation of the dissociation constant of H2SO4
    calc_lnkHSO4 = LOG(10**(647.59/T - 6.3451 + 0.019085*T - 0.5208*I_p05))
  END FUNCTION calc_lnkHSO4
! ****************************
  

! *************************
! *** Calculate ln(KSi) ***
! *************************
  FUNCTION calc_lnkSi(T,T_ln,I,I_p05)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnkSi
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_ln,I,I_p05
! calculation of the dissociation constant of SiO2
    calc_lnkSi = 117.40 - 8904.2/T - 19.334*T_ln &
                + (3.5913 - 458.79/T)*I_p05 + (-1.5998 + 188.74/T)*I + (0.07871 - 12.1652/T)*I*I
  END FUNCTION calc_lnkSi
! *************************


! *****************************
! *** Calculate log10(Kcal) ***
! *****************************
  FUNCTION calc_logkcal(T,T_log,S,S_p05,S_p15)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_logkcal
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_log,S,S_p05,S_p15
! Calculation of the solubility constant of calcite.
! return function value
    calc_logkcal = -171.9065 - 0.077993*T + 2839.319/T + 71.595*T_log &
                  + (-0.77712 + 0.0028426*T + 178.34/T)*S_p05 - 0.07711*S + 0.0041249*S_p15
  END FUNCTION calc_logkcal
! ***************************


! *****************************
! *** Calculate log10(Karg) ***
! *****************************
  FUNCTION calc_logkarg(T,T_log,S,S_p05,S_p15)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_logkarg
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,T_log,S,S_p05,S_p15
! Calculation of the solubility constant of aragonite.
    calc_logkarg = -171.945 - 0.077993*T + 2903.293/T + 71.595*T_log &
                  + (-0.068393 + 0.0017276*T + 88.135/T)*S_p05 - 0.10018*S + 0.0059415*S_p15
  END FUNCTION calc_logkarg
! **************************
  

! ***************************
! *** Calculate ln(Q_CO2) ***
! ***************************
  FUNCTION calc_lnQ_CO2(T,S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnQ_CO2
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,S
! Henry's Law constant for CO2 with concentrations as moles per kg of solution [Weiss, 1974].
! The dependence of the solubility coefficient on hydrostatic pressure is ignored.
    calc_lnQ_CO2 = -60.2409 + 93.4517*(100/T) + 23.3585*LOG(T/100) &
      & + S*(0.023517 - 0.023656*(T/100) + 0.0047036*(T/100)**2)
  END FUNCTION calc_lnQ_CO2
! ***************************
  

! ***************************
! *** Calculate ln(Q_O2) ***
! ***************************
  FUNCTION calc_lnQ_O2(T,S)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_lnQ_O2
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,S
! Henry's Law constant for O2 with concentrations as moles per kg of solution.
! Adapted from a formula from Millero and Sohn [1992] giving O2 solubility in units of umol kg-1,
! back to a Henry's Law constant for O2 in units of mol kg-1 atm-1.
! The dependence of the solubility coefficient on hydrostatic pressure is ignored.
    calc_lnQ_O2 = -173.9894 + 255.5907*(100/T) + 146.4813*LOG(T/100) - 22.2040*(T/100) &
      & + S*(-0.037362 + 0.016504*(T/100) - 0.0020564*(T/100)**2) &
      & - lOG(1.0E6) - LOG(0.20946)
  END FUNCTION calc_lnQ_O2
! ***************************


! *****************************************
! *** Calculate pressure effects factor ***
! *****************************************
  FUNCTION corr_p(Tc,P,RT,dp)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::corr_p
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::Tc,P,RT
    REAL(KIND=fp_1),INTENT(IN),DIMENSION(5)::dp
! return function value
    corr_p = (-(dp(1) + dp(2)*Tc + dp(3)*Tc*Tc) + (5.0E-4*(dp(4) + dp(5)*Tc))*P)*P/RT
  END FUNCTION corr_p
! *****************************************
  

! ********************************************
! *** Calculate calcite compensation depth ***
! ********************************************
  FUNCTION calc_D_calsat(T,S,conc_Ca,conc_CO3)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_D_calsat
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,S
    REAL(KIND=fp_1),INTENT(IN)::conc_Ca,conc_CO3
! local variables
    REAL(KIND=fp_1)::a,b,c,Tc,RT,T_log,S_p05,S_p15,kcal_uncorr
    REAL(KIND=fp_1),DIMENSION(2)::satD=0.0
! calculate local constants
    S_p15      = S**1.5
    S_p05      = S**0.5
    T_log      = LOG10(T)
    Tc = T-const_zeroC
    RT = const_R*T
! calculate the non-pressure-corrected value of kcal
    kcal_uncorr = 10**(calc_logkcal(T,T_log,S,S_p05,S_p15))
! solve lysocline equation and find both roots
! 'a' <->  (5.0E-4*(dp(4) + dp(5)*Tc))/RT
! 'b' <->  -(dp(1) + dp(2)*Tc + dp(3)*Tc*Tc)/RT
! 'c' <-> -ln(conc_Ca*conc_CO3/kcal)
    a = (5.0E-4*(dpCaCO3cal(4) + dpCaCO3cal(5)*Tc))/RT
    b = -(dpCaCO3cal(1) + dpCaCO3cal(2)*Tc + dpCaCO3cal(3)*Tc*Tc)/RT
    c = -LOG(conc_Ca*conc_CO3/kcal_uncorr)
!    PRINT*,"a: ",a
!    PRINT*,"b: ",b
!    PRINT*,"c: ",c
!
    satD(1) = (-b + (b**2 - 4*a*c)**0.5)/(2*a)
    satD(2) = (-b - (b**2 - 4*a*c)**0.5)/(2*a)
! return function value
! NOTE: remember to convert pressure to an estimate of depth (i.e., introduce a factor of 10.0)
    calc_D_calsat = 10.0 * satD(1)
  END FUNCTION calc_D_calsat
! ********************************************

! **********************************************
! *** Calculate aragonite compensation depth ***
! **********************************************
  FUNCTION calc_D_argsat(T,S,conc_Ca,conc_CO3)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_D_argsat
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,S
    REAL(KIND=fp_1),INTENT(IN)::conc_Ca,conc_CO3
! local variables
    REAL(KIND=fp_1)::a,b,c,Tc,RT,T_log,S_p05,S_p15,karg_uncorr
    REAL(KIND=fp_1),DIMENSION(2)::satD=0.0
! calculate local constants
    S_p15      = S**1.5
    S_p05      = S**0.5
    T_log      = LOG10(T)
    Tc = T-const_zeroC
    RT = const_R*T
! calculate the non-pressure-corrected value of karg
    karg_uncorr = 10**(calc_logkarg(T,T_log,S,S_p05,S_p15))
! solve lysocline equation and find both roots
! 'a' <->  (5.0E-4*(dp(4) + dp(5)*Tc))/RT
! 'b' <->  -(dp(1) + dp(2)*Tc + dp(3)*Tc*Tc)/RT
! 'c' <-> -ln(conc_Ca*conc_CO3/karg)
    a = (5.0E-4*(dpCaCO3arg(4) + dpCaCO3arg(5)*Tc))/RT
    b = -(dpCaCO3arg(1) + dpCaCO3arg(2)*Tc + dpCaCO3arg(3)*Tc*Tc)/RT
    c = -LOG(conc_Ca*conc_CO3/karg_uncorr)
!    PRINT*,"a: ",a
!    PRINT*,"b: ",b
!    PRINT*,"c: ",c
!
    satD(1) = (-b + (b**2 - 4*a*c)**0.5)/(2*a)
    satD(2) = (-b - (b**2 - 4*a*c)**0.5)/(2*a)
! return function value
! NOTE: remember to convert pressure to an estimate of depth (i.e., introduce a factor of 10.0)
    calc_D_argsat = 10.0 * satD(1)
  END FUNCTION calc_D_argsat
! **********************************************


! ******************************************
! *** Convert pH scales [H+]T to [H+]SWS ***
! ******************************************
  FUNCTION pH_TtoSWS(T,I,I_p05,kF,kHSO4,c_F,c_SO4)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::pH_TtoSWS
! dummy arguments
    REAL(KIND=fp_1),INTENT(IN)::T,I,I_p05,kF,kHSO4,c_F,c_SO4
! Convert between total hydrogen ion pH scale (T) and seawater pH scale (SWS).
! [Millero, 1995]
    pH_TtoSWS = LOG((1 + kHSO4*c_SO4 + kF*c_F)/(1 + kHSO4*c_SO4))
  END FUNCTION pH_TtoSWS
! *******************************************
 

! ********************************
! *** Calculate pH (NBS scale) ***
! ********************************
  FUNCTION calc_pH_NBS(const,S,tot_alk,sum_CO2,tot_PO4,tot_Si)
    IMPLICIT NONE
! result variable
    REAL(KIND=fp_1)::calc_pH_NBS
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(10)::const
    REAL(KIND=fp_1),INTENT(IN)::S,tot_alk,sum_CO2,tot_PO4,tot_Si
! local variables
    REAL(KIND=fp_1)::tot_B,tot_Ca
    REAL(KIND=fp_1)::k,k1,k2,kB,kW,kSi,zed
    REAL(KIND=fp_1)::CO2_alk,c_CO2,c_CO3,c_HCO3
    REAL(KIND=fp_1)::c_H,c_H_old,c_H1,c_H2
! estimate borate and Ca concentrations (from salinity) and pressure (from depth in m)
    tot_B = calc_B(S)
    tot_Ca  = calc_Ca(S)
! copy imported constants
    k     = const(1)
    k1    = const(2)
    k2    = const(3)
    kB    = const(4)
    kW    = const(5)
    kSi   = const(6)
! pH calculation loop
    c_H = 7E-9    
    DO     
      c_H_old=c_H   
      CO2_alk = tot_alk - (kW/c_H - c_H) &
	- ((kB*tot_B)/(c_H+kB)) &
	- ((kSi*tot_Si)/(c_H+kSi)) &
	- 1.1*tot_PO4   
      zed =     ((4*CO2_alk + sum_CO2*k - CO2_alk*k)**2 + 4*(k - 4)*CO2_alk**2)**0.5
      c_HCO3 =  (sum_CO2*k - zed)/(k - 4)
      c_CO3 =   (CO2_alk*k - sum_CO2*k - 4*CO2_alk + zed)/(2*(k - 4))
      c_CO2 =   sum_CO2 - CO2_alk + (CO2_alk*k - sum_CO2*k - 4*CO2_alk + zed)/(2*(k - 4))        
      c_H1 =  k1*c_CO2/c_HCO3
      c_H2 =  k2*c_HCO3/c_CO3
      c_H =   SQRT(c_H1*c_H2)    
      IF (ABS(1 - c_H/c_H_old) < 0.001) EXIT   
    END DO
! return function value
    calc_pH_NBS = -LOG10(c_H)
  END FUNCTION calc_pH_NBS
! ********************************


END MODULE sue_co2calc
