! *************************************************************************************************
! sue_sedcor.f90
! User-defined sediment core system routines
! *************************************************************************************************



MODULE sue_sedcor


  USE sue_config
  USE sue_lib
  USE sue_co2calc
  USE sue_sed_lib
  IMPLICIT NONE
  SAVE
  
  
CONTAINS
  

! ****************************************************
! *** CALCULATE REMINERALIZATION FUNCTION PROFILES ***
! ****************************************************
  SUBROUTINE calc_remin_frac_sedcor(tracer,sedcortracer,phys,sedcorpar)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_tracer)::tracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_tracer)::sedcortracer
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
! local variables
    INTEGER::c,l,m,p
    INTEGER::sed_lay_mid           ! mid-sediment/ocean adjacent ocean layer number
    REAL(KIND=fp_1),DIMENSION(n_lay)::sed_lay_frac ! sediment/ocean layer fractional adjacencies
    REAL(KIND=fp_1)::sed_lay_Dtop  ! sediment/ocean adjacent top layer depth
    REAL(KIND=fp_1)::sed_lay_Dbot  ! sediment/ocean adjacent bottom layer depth
    REAL(KIND=fp_1)::sed_D_calsat  ! ocean calcite saturation depth w.r.t. sedimentary layer
    REAL(KIND=fp_1)::sed_D_argsat  ! ocean aragonite saturation depth w.r.t. sedimentary layer
    REAL(KIND=fp_1)::delta_Dargsat ! depth below Dargsat (m)
    REAL(KIND=fp_1)::av_Dargsat    ! mean (mass-weighted) aragonite saturation depth (m)
    REAL(KIND=fp_1)::arg_remin_top ! value of the arg remin function at ocean layer top depth
    REAL(KIND=fp_1)::arg_remin_bot ! value of the arg remin function at ocean layer bottom depth
    REAL(KIND=fp_1)::arg_remin_sed ! value of the arg remin function at sediment layer (mid) depth
    REAL(KIND=fp_1)::et_opal_Ganan ! opal remin e-folding time using the Gnanadesikan [1999] relationship
    REAL(KIND=fp_1)::lay_res       ! residence time within ocean layer (d)
    REAL(KIND=fp_1)::lay_Si_eq     ! equilibrium [H4SiO4] within ocean layer (umol kg-1)
    REAL(KIND=fp_1)::lay_u         ! degree of undersaturation (w.r.t. opal) within ocean layer
    REAL(KIND=fp_1)::lay_D         ! mean depth within ocean layer (m)
    REAL(KIND=fp_1)::lay_vel       ! settling velocity within ocean layer (m d-1)
    
! *** calculate water column remineralization above user-defined sediment cores ***
    
! initialize variable default values
    sedcorpar(:,ised_par_reminf_Corg) = 1.0
    sedcorpar(:,ised_par_reminf_opal) = 1.0
    sedcorpar(:,ised_par_reminf_cal)  = 1.0
    sedcorpar(:,ised_par_reminf_arg)  = 1.0

    IF (runopt(ropt_sedcor) == yes) THEN
      
      DO p = 1,n_sedcor
	
! set ocean column numbers
	c = INT(sedcorpar(p,ised_par_colnum))
! (1) Corg
	SELECT CASE (bioopt(bopt_remin_Corg_alt))
	CASE ("1")
!       scheme #1 : exponential scheme
	  IF (sedcorpar(p,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedcorpar(p,ised_par_reminf_Corg) = par_remin_fracrecal_Corg + &
	      & (1.0 - par_remin_fracrecal_Corg) * &
	      & EXP(-(sedcorpar(p,ised_par_D_mid) - phys(c,1,ip_remin_topD)) / par_remin_eL_Corg)
	  ELSE
	    sedcorpar(p,ised_par_reminf_Corg) = 1.0
	  ENDIF
	CASE ("2")
!       scheme #2 : 
	  IF (sedcorpar(p,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedcorpar(p,ised_par_reminf_Corg) =  par_remin_fracrecal_Corg +                   &
	      & (1.0 - par_remin_fracrecal_Corg) *                                            &
	      & (                                                                             &
	      &   (par_remin_pL_Corg + sedcorpar(p,ised_par_D_mid) - phys(c,1,ip_remin_topD)) &
	      &   /                                                                           &
	      &   par_remin_pL_Corg                                                           &
	      & )**par_remin_pP_Corg
	  ELSE
	    sedcorpar(p,ised_par_reminf_Corg) = 1.0
	  ENDIF
	END SELECT
! (2) opal
	SELECT CASE (bioopt(bopt_remin_opal_alt))
	CASE ("1")
!       scheme #1 : exponential scheme
	  IF (sedcorpar(p,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedcorpar(p,ised_par_reminf_opal) = &
	      & EXP(-(sedcorpar(p,ised_par_D_mid) - phys(c,1,ip_remin_topD)) / par_remin_eL_opal)
	  ELSE
	    sedcorpar(p,ised_par_reminf_opal) = 1.0
	  ENDIF
	CASE ("2")
!       scheme #2 : 
	  IF (sedcorpar(p,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedcorpar(p,ised_par_reminf_opal) = 1.0
	    DO l = 1,n_lay
	      IF (phys(c,l,ip_D_bot) < sedcorpar(p,ised_par_D_mid)) THEN
		IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
		  lay_res = (phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)) / par_remin_opal_vel
		  et_opal_Ganan = -1.0 / (LOG(1.0 - 1.32E+16 * EXP(-11481.0 / tracer(c,l,it_T))))
		  sedcorpar(p,ised_par_reminf_opal) = sedcorpar(p,ised_par_reminf_opal) * &
		    & EXP(-lay_res / et_opal_Ganan)
		  IF (sedcorpar(p,ised_par_reminf_opal) < 0.0) sedcorpar(p,ised_par_reminf_opal) = 0.0
		ELSE
		  sedcorpar(p,ised_par_reminf_opal) = 1.0
		END IF
	      ELSE
		lay_res = (sedcorpar(p,ised_par_D_mid) - phys(c,l,ip_D_top)) / par_remin_opal_vel
		et_opal_Ganan = -1.0 / (LOG(1.0 - 1.32E+16 * EXP(-11481.0 / sedcortracer(p,it_T))))
		sedcorpar(p,ised_par_reminf_opal) = sedcorpar(p,ised_par_reminf_opal) * &
		  & EXP(-lay_res / et_opal_Ganan)
		IF (sedcorpar(p,ised_par_reminf_opal) < 0.0) sedcorpar(p,ised_par_reminf_opal) = 0.0
		EXIT
	      ENDIF
	    END DO
	  ELSE
	    sedcorpar(p,ised_par_reminf_opal) = 1.0
	  ENDIF
	CASE ("3")
!       scheme #3 : 
	  IF (sedcorpar(p,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedcorpar(p,ised_par_reminf_opal) = 1.0
	    DO l = 1,n_lay
	      IF (phys(c,l,ip_D_bot) < sedcorpar(p,ised_par_D_mid)) THEN
		IF (phys(c,1,ip_remin_topD) < phys(c,l,ip_D_bot)) THEN
		  lay_Si_eq = conv_umol_mol * 10.0**(6.44 - 968.0 / tracer(c,l,it_T))
		  lay_u     = (lay_Si_eq - tracer(c,l,it_Si)) / lay_Si_eq
		  IF (lay_u > 1.0) lay_u = 1.0
		  IF (lay_u < 0.0) lay_u = 0.0
		  IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
		    lay_res = (phys(c,l,ip_D_bot) - phys(c,1,ip_remin_topD)) / par_remin_opal_vel
		  ELSE
		    lay_res = (phys(c,l,ip_D_bot) - phys(c,l,ip_D_top)) / par_remin_opal_vel
		  END IF
		  sedcorpar(p,ised_par_reminf_opal) = sedcorpar(p,ised_par_reminf_opal) *                &
		    & (1.0 -                                                                             &
		    &   lay_res * par_remin_opal_K *                                                     &
		    &   (1.0 / 0.71) *                                                                   &
		    &   (                                                                                &
		    &     (0.16 * (1.0 + (tracer(c,l,it_T) - const_zeroC) / 15.0) * lay_u) +             &
		    &     (0.55 * ((1.0 + (tracer(c,l,it_T) - const_zeroC) / 400.0)**4.0 * lay_u)**9.25) &
		    &   )                                                                                &
		    & )
		  IF (sedcorpar(p,ised_par_reminf_opal) < 0.0) sedcorpar(p,ised_par_reminf_opal) = 0.0
		ELSE
		  sedcorpar(p,ised_par_reminf_opal) = 1.0
		END IF
	      ELSE
		lay_Si_eq = conv_umol_mol * 10.0**(6.44 - 968.0 / sedcortracer(p,it_T))
		lay_u     = (lay_Si_eq - sedcortracer(p,it_Si)) / lay_Si_eq
		IF (lay_u > 1.0) lay_u = 1.0
		IF (lay_u < 0.0) lay_u = 0.0
		IF (phys(c,1,ip_remin_topD) > phys(c,l,ip_D_top)) THEN
		  lay_res = (sedcorpar(p,ised_par_D_mid) - phys(c,1,ip_remin_topD)) / par_remin_opal_vel
		ELSE
		  lay_res = (sedcorpar(p,ised_par_D_mid) - phys(c,l,ip_D_top)) / par_remin_opal_vel
		END IF
		sedcorpar(p,ised_par_reminf_opal) = sedcorpar(p,ised_par_reminf_opal) *                    &
		  & (1.0 -                                                                                 &
		  &   lay_res * par_remin_opal_K *                                                         &
		  &   (1.0 / 0.71) *                                                                       &
		  &   (                                                                                    &
		  &     (0.16 * (1.0 + (sedcortracer(p,it_T) - const_zeroC) / 15.0) * lay_u) +             &
		  &     (0.55 * ((1.0 + (sedcortracer(p,it_T) - const_zeroC) / 400.0)**4.0 * lay_u)**9.25) &
		  &   )                                                                                    &
		  & )
		IF (sedcorpar(p,ised_par_reminf_opal) < 0.0) sedcorpar(p,ised_par_reminf_opal) = 0.0
		EXIT
	      ENDIF
	    END DO
	  ELSE
	    sedcorpar(p,ised_par_reminf_opal) = 1.0
	  ENDIF
	END SELECT
! (3) calcite
	SELECT CASE (bioopt(bopt_remin_cal_alt))
	CASE ("1")
!       scheme #1 : none
	  sedcorpar(p,ised_par_reminf_cal)  = 1.0
	CASE ("2")
!       scheme #2 : 
	  IF (sedcorpar(p,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedcorpar(p,ised_par_reminf_cal) = par_remin_fracrecal_cal + &
	      & (1.0 - par_remin_fracrecal_cal) *                        &
	      & EXP(-(sedcorpar(p,ised_par_D_mid) - phys(c,1,ip_remin_topD)) / par_remin_eL_cal)
	  ELSE
	    sedcorpar(p,ised_par_reminf_cal) = 1.0
	  ENDIF
	END SELECT
! (4) aragonite
!     calculate aragonite saturation depths
	SELECT CASE (bioopt(bopt_remin_arg_alt))
	CASE ("1")
!       scheme #1 : none
	  sedcorpar(p,ised_par_reminf_arg)  = 1.0
	CASE ("2")
!       scheme #2 : exponential scheme
	  sed_D_argsat = phys(c,l,ip_D_argsat)
	  IF (sedcorpar(p,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    sedcorpar(p,ised_par_reminf_arg)  = &
	      & EXP(-(sedcorpar(p,ised_par_D_mid) - sed_D_argsat) / par_remin_eL_arg)
	  ELSE
	    sedcorpar(p,ised_par_reminf_arg)  = 1.0
	  ENDIF
	CASE ("3")
!       scheme #3 : 
	  IF (sedcorpar(p,ised_par_D_mid) > phys(c,1,ip_remin_topD)) THEN
	    av_Dargsat =                                           &
	      & SUM                                                &
	      & (                                                  &
	      &   phys(c,par_remin_nlbase + 1:n_lay,ip_D_argsat) * &
	      &   phys(c,par_remin_nlbase + 1:n_lay,ip_M)          &
	      & )                                                  &
	      & /                                                  &
	      & SUM(phys(c,par_remin_nlbase + 1:n_lay,ip_M))
	    delta_Dargsat = sedcorpar(p,ised_par_D_mid) - av_Dargsat
	    IF (delta_Dargsat < 0.0) THEN
	      sedcorpar(p,ised_par_reminf_arg) = 1.0
	    ELSE
	      sedcorpar(p,ised_par_reminf_arg) = 1.0 - &
		& (delta_Dargsat / 11000.0)**0.25 * (1.0 - EXP(-delta_Dargsat / 1400.0))**4.5
	    ENDIF
	    IF (sedcorpar(p,ised_par_reminf_arg) < 0.0) sedcorpar(p,ised_par_reminf_arg) = 0.0
	  ELSE
	    sedcorpar(p,ised_par_reminf_arg)  = 1.0
	  ENDIF
	END SELECT
	
      ENDDO
      
    ENDIF
    
  END SUBROUTINE calc_remin_frac_sedcor
! ****************************************************


! *************************************************************
! *** initialize sediment cores: sedimentary layers content ***
! *************************************************************
  SUBROUTINE init_sedcor_layers_default(sedcor,sedcortop)
    IMPLICIT NONE
! dummy variables
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sedcor_tot,n_sed_com_par)::sedcor
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_com_par)::sedcortop
! local variables
    INTEGER::o,p
! start of sediment core number loop
    DO p = 1,n_sedcor
! set default sediment stack values
! NOTE: sediment component volumes are in the units of 
!       actual volume of solid matter per cm2 area of sub-layer
! NOTE: initialize total sediment solids age tied to refractory volume
!       (as it comprises 100% of sediment on initialization)
      sedcortop(p,:)               = 0.0
      sedcortop(p,ised_refrac)     = (1.0 - par_sedcor_poros_top) * par_sedcor_top_th
      DO o = 1,n_sedcor_tot_init
	sedcor(p,o,:)               = 0.0
	sedcor(p,o,ised_refrac)     = (1.0 - par_sedcor_poros)
      END DO
      DO o = (n_sedcor_tot_init + 1),n_sedcor_tot
	sedcor(p,o,:) = 0.0
      END DO
    END DO
  END SUBROUTINE init_sedcor_layers_default
! *************************************************************


! *******************************************
! *** initialize sediment core parameters ***
! *******************************************
  SUBROUTINE init_sedcor_par(sedcorpar,phys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::c,p

! *** set default sediment depths ***
! start of loop
    DO p = 1,n_sedcor
! set height of top layer of old sediment
      sedcorpar(p,ised_par_top) = REAL(n_sed_tot_init,KIND=fp_1)
! NOTE: set the top and bottom depths of the sediment core slightly offset from the mid depth,
!       to avoid possible divide-by-zero errors elsewhere in the program,
!       despite the fact that the sediment core layers do not actually span a non-zero ocean depth interval
      sedcorpar(p,ised_par_D_mid) = sedcorpar(p,ised_par_D_PIG_mid)
      sedcorpar(p,ised_par_D_top) = sedcorpar(p,ised_par_D_mid) - 0.001
      sedcorpar(p,ised_par_D_bot) = sedcorpar(p,ised_par_D_mid) + 0.001
    END DO

! *** set default sediment core 'virtual' area
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      sedcorpar(p,ised_par_A) = phys(c,1,ip_A_bot)
    END DO

  END SUBROUTINE init_sedcor_par
! *************************************************


! *************************************************************************
! *** update user-defined sediment core bathymetry from sealevel change ***
! *************************************************************************
  SUBROUTINE update_sedcor_bath(sedcorpar,phys,sealevel)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(in)::sealevel
! local variables
    INTEGER::c,p

! *** calculate new depths taking into account sea-level changes ***
! NOTE: sea-level falls are represented by a NEGATIVE value
! NOTE: the depth below sealevel of the top of the uppermost sediment layer can go negative!
! NOTE: set the top and bottom depths of the sediment core slightly offset from the mid depth,
!       to avoid possible divide-by-zero errors elsewhere in the program,
!       despite the fact that the sediment core layers do not actually span a non-zero ocean depth interval
    sedcorpar(:,ised_par_D_mid) = sedcorpar(:,ised_par_D_PIG_mid) + sealevel
    sedcorpar(:,ised_par_D_top) = sedcorpar(:,ised_par_D_mid) - 0.001
    sedcorpar(:,ised_par_D_bot) = sedcorpar(:,ised_par_D_mid) + 0.001
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      sedcorpar(p,ised_par_A) = phys(c,1,ip_A_bot)
    END DO

  END SUBROUTINE update_sedcor_bath
! *************************************************************************


! *****************************************************************************
! *** calculate user-defined sediment core sediment/ocean layer adjacencies ***
! *****************************************************************************
  SUBROUTINE calc_sedcor_lay(sedcorpar,phys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::c,l,p
    
! *** find ocean layers adjactent to each sediment layer ***
! NOTE: the top and bottom ocean layer numbers in the sediment parameter array are real numbers,
!       but are used for storing an integer value
! NOTE: the ocean column number is an integer, but is stored as a real in the sediment parameter array
    DO p = 1,n_sedcor
! (1) determine ocean column number
      c = INT(sedcorpar(p,ised_par_colnum))
! (2) zero ocean layer elements
      sedcorpar(p,ised_par_lay_top) = 0.0
      sedcorpar(p,ised_par_lay_bot) = 0.0
! (3) work down through water column:
! the first ocean layer that has a D_bot value >= the sediment D_mid value,
! must be the ocean layer that affects that particular sediment layer
      DO l = 1,n_lay
	IF (phys(c,l,ip_D_bot) >= sedcorpar(p,ised_par_D_mid)) THEN
	  sedcorpar(p,ised_par_lay_bot) = REAL(l,KIND=fp_1)
	  sedcorpar(p,ised_par_lay_top) = sedcorpar(p,ised_par_lay_bot)
	  EXIT
	END IF
      END DO
    END DO
    
  END SUBROUTINE calc_sedcor_lay
! *****************************************************************************


! ***************************************************************************
! *** average tracer and phys values at user-defined sediment core depths ***
! ***************************************************************************
  SUBROUTINE calc_sedcor_tracer(sedcorpar,sedcortracer,tracer,phys)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_tracer)::sedcortracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_tracer)::tracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::c,l,p,it

! *** calculate average tracer concentrations (and water temperature) for each sediment depth ***
! loop through all sediment cores
! NOTE: the ocean column number is an integer, but is stored as a real in the sediment parameter array
    DO p = 1,n_sedcor
! (1) determine ocean column and layer numbers
      c = INT(sedcorpar(p,ised_par_colnum))
      l = INT(sedcorpar(p,ised_par_lay_top))
! (2) set the tracer concentration the corresponding ocean layer
      DO it = 1,n_tracer
	sedcortracer(p,it) = tracer(c,l,it)
      ENDDO
! (3) set the physical characteristics of the corresponding ocean layer
      sedcorpar(p,ised_par_T) = phys(c,l,ip_T)
    ENDDO

  END SUBROUTINE calc_sedcor_tracer
! ***************************************************************************


! ****************************************************************************
! *** carbonate equilibrium constants at user-defined sediment core depths ***
! ****************************************************************************
  SUBROUTINE calc_sedcor_disconst(sedcordis,sedcorpar,sedcortracer)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_dis)::sedcordis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_tracer)::sedcortracer
! local variables
    INTEGER::p
    
! *** calculate carbonate equilibrium constants for each sediment depth ***
! loop through all sediment cores,
! calculate carbonate equilibrium constants at mid sediment depths and return function value
    DO p = 1,n_sedcor
      CALL co2const_ex(sedcorpar(p,ised_par_T),sedcortracer(p,it_S),sedcorpar(p,ised_par_D_mid), &
	& sedcordis(p,:))
    ENDDO
    
  END SUBROUTINE calc_sedcor_disconst
! ****************************************************************************


! ********************************************************
! *** calculate sediment surface carbonate equilibrium ***
! ********************************************************
  SUBROUTINE calc_sedcor_carbchem(sedcorpar,sedcordis,sedcortracer)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_dis)::sedcordis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_tracer)::sedcortracer
! local variables
    INTEGER::p
    REAL(KIND=fp_1)::fug_CO2,conc_CO2,conc_HCO3
    
! *** calculate carbonate equilibria for each sediment depth ***
! loop through all sediment depths,
! and calculate 'ohm_cal', 'ohm_arg', 'conc_Ca', 'conc_CO3' and pH values
    DO p = 1,n_sedcor
      CALL calccarb_ex(sedcordis(p,:), &
	& sedcortracer(p,it_S), &
	& sedcortracer(p,it_alk), &
	& sedcortracer(p,it_CO2), &
	& sedcortracer(p,it_PO4), &
	& sedcortracer(p,it_Si), &
	& sedcorpar(p,ised_par_Ca), &
	& fug_CO2, &
	& conc_CO2, &
	& sedcorpar(p,ised_par_CO3), &
	& conc_HCO3, &
	& sedcorpar(p,ised_par_ohmcal), &
	& sedcorpar(p,ised_par_ohmarg), &
	& sedcorpar(p,ised_par_dCO3cal), &
	& sedcorpar(p,ised_par_dCO3arg), &
	& sedcorpar(p,ised_par_pH))
    END DO

  END SUBROUTINE calc_sedcor_carbchem
! ********************************************************


! ****************************************************************************
! *** couple in user-defined sediment cores to ocean-atmosphere sub-system ***
! ****************************************************************************
  SUBROUTINE couple_sedcor(R_dt,I_tice,I_tloop,tracer,sedcortracer,sedcorfnp,frefrac,phys, &
    & sedcor,sedcortop,sedcorpar,sedmixk,dtot,sedcorfdis,sscc)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in)::R_dt ! primary model time step as a REAL (in yrs)
    INTEGER,INTENT(in)::I_tice
    INTEGER,INTENT(in)::I_tloop
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_col,n_lay,n_tracer)::tracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_tracer)::sedcortracer
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sed_fnp)::sedcorfnp
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col)::frefrac
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sedcor_tot,n_sed_com_par)::sedcor
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_com_par)::sedcortop
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(0:n_sed_mix)::sedmixk
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_tracer)::dtot
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_com_par)::sedcorfdis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_sscc)::sscc
! local variables
    INTEGER::c,l,p,o
    INTEGER::n_sedcor_stack_top          ! sediment stack top (incomplete) layer number
    REAL(KIND=fp_1)::sedcor_A_cm2        ! sedimentary layer area in cm2
    REAL(KIND=fp_1)::R_tice              ! simulation year as a REAL (kyrs)
    REAL(KIND=fp_1)::R_tloop             ! spin-up year as a REAL (kyrs)
    REAL(KIND=fp_1)::new_sed_vol         ! new sediment volume (as SOLID material)
    REAL(KIND=fp_1)::exe_sed_vol         ! exchangable sediment volume (as SOLID material)
    REAL(KIND=fp_1)::top_sed_vol         ! top sediment volume (as SOLID material)
    REAL(KIND=fp_1)::exe_sed_th          ! exchangable sediment thickness (as stack material)
    REAL(KIND=fp_1)::sed_top_th          ! sediment top ('well-mixed') layer thickness
    REAL(KIND=fp_1)::dsed_top_th         ! change in sediment top ('well-mixed') layer thickness
    REAL(KIND=fp_1)::sedcor_stack_top_th ! sediment stack top (incomplete) sub-layer thickness
    REAL(KIND=fp_1)::sed_FeO_fdis        ! FeO dissolution flux (cm3 cm-2 a-1)
    REAL(KIND=fp_1),DIMENSION(n_sed_com_par)::new_sed ! new (sedimenting) top layer material
    REAL(KIND=fp_1),DIMENSION(n_sed_com_par)::dis_sed ! remineralized top layer material
    REAL(KIND=fp_1),DIMENSION(n_sed_com_par)::exe_sed ! top layer material to be exchanged with stack
    
! NOTE: the top ('well-mixed') sediment layer is par_sedcor_top_th cm in thickness,
!       the sediment composition components sum to par_sedcor_top_th (cm3 cm-2) in this layer, and
!       the porosity is par_sedcor_poros_top (i.e., solids actually occupy 1.0 - par_sedcor_poros_top)


! *** initialize local variables ***
! zero working sediment arrays
    new_sed(:) = 0.0
    dis_sed(:) = 0.0
    exe_sed(:) = 0.0
! set local constants
    R_tice           = REAL(I_tice,KIND=fp_1)
    R_tloop          = REAL(I_tloop,KIND=fp_1)

    
! *** interact ocean boxes and sediment sub-system ***
! NOTE: the overall scheme is to loop through each ocean water column and each sediment layer sub-system, and
!       (a) calculate local constants
!       (b) calculate new sedimenting material to be added to the sediment top ('well mixed') layer, and
!           calculate the thickness of material that this represents
!       (c) estimate dissolution from sediment top layer
!       (d) update sediment top and sediment stack:
!           - add new sedimenting material to sediment top layer
!           - remove remineralized material
!           calculate potential thickness of sediment top layer including new sedimenting material:
!           - remove material to the sediment stack below if the thickness is >= par_sedcor_top_th cm, or
!           - add material from the sediment stack below if the thickness is < par_sedcor_top_th cm, then
!           update sediment stack height
!       (e) mix the sediment stack if this is required
!       (f) check the thickness of sediment stack:
!           if it is within 1.0 cm of the maximum thickness, then remove the bottom n_sedcor_tot_init cm
!           entirely, and move the remaining sedimentary layers down to start at the bottom of the stack


! *** sediment core loop ***
    DO p = 1,n_sedcor

! set ocean column number
      c = INT(sedcorpar(p,ised_par_colnum))
! set ocean layer number number
      l = INT(sedcorpar(p,ised_par_lay_top))

!print*,"(a)"
! ***** (a) calculate local constants
! calculate local loop constants
! NOTE: the total sediment area for user-defined sediment cores is assumed to be 
!       the area of the bottom of the mixed layer 
!       (which is the total area over which new production is exported)
      sedcor_A_cm2 = sedcorpar(p,ised_par_A) * conv_m2_cm2

!print*,"(b)"
! ***** (b) calculate new sedimenting material to be added to the sediment top layer
      IF (sedopt(sopt_sedcor_fixedacc) == no) THEN
!           new sediment determined by model-calculated water column flux
!           NOTE: sedimentary material is represented as SOILD matter (i.e., zero porosity)
!           NOTE: convert new particulate matter added to sediments (in mol yr-1) to (cm3 cm-2 yr-1)
!           NOTE: sedimenting FeO is assumed to be made up of Fe in POM, Fe scavenged by falling POM, 
!                 and the fraction of Fe in terrigeneous dust which didn't dissolved in the surface ocean
!           NOTE: convert refractory (terregeneous dust) flux from (g cm-2 yr-1) to (cm3 cm-2 yr-1)
!           NOTE: the additional refractory flux must be converted from (g cm-1 kyr-1) to (cm3 cm-2 yr-1)
!           NOTE: the initial fraction of Fe in the terrigeneous dust flux is first deducted from 
!                 the total flux, to give the refractory flux
!           NOTE: T-dependent fractionation after Mook [1986],
!                 otherwise [CO32-]-dependent fractionation adapted from Spero et al. [1997]
!           NOTE: the Mook [1986] fractionation is approximated by assuming that 
!                 the d13C of HCO3- is the same as that of total DIC 
!                 (this is in order to be consistent with the calculation of benthic fractionation where 
!                 the fractionation within the aqueous carbonate system is not explicitly calculated)
!           NOTE: O. universa relationship is taken to be the average of the dark and high-light 
!                 regressions resented by Spero et al. [1997] under constant total DIC conditions
!           NOTE: G. bulloides relationship is taken to be the average of the 12th and 13th chamber 
!                 regressions resented by Spero et al. [1997] under constant total DIC conditions
!           NOTE: the regression have been corrected for the d13C of total DIC used by Spero et al. [1997]
!           calculate 'solid' components
	new_sed(ised_Corg)       = sedcorfnp(p,ised_fnp_Corg)     * conv_Corg_mol_cm3  / sedcor_A_cm2
	new_sed(ised_FeO)        = sedcorfnp(p,ised_fnp_Feorg)    * conv_FeO_mol_cm3   / sedcor_A_cm2
	new_sed(ised_cal)        = sedcorfnp(p,ised_fnp_cal)      * conv_cal_mol_cm3   / sedcor_A_cm2
	new_sed(ised_arg)        = sedcorfnp(p,ised_fnp_arg)      * conv_arg_mol_cm3   / sedcor_A_cm2
	new_sed(ised_opal)       = sedcorfnp(p,ised_fnp_opal)     * conv_opal_mol_cm3  / sedcor_A_cm2
	new_sed(ised_refrac)     = (frefrac(c) + par_frefrac)       * conv_refrac_g_cm3
!           calculate new sediment (bulk material) stable isotopes
	new_sed(ised_13C_Corg)   = sedcorfnp(p,ised_fnp_13C_Corg) * conv_Corg_mol_cm3  / sedcor_A_cm2
	new_sed(ised_13C_cal)    = sedcorfnp(p,ised_fnp_13C_cal)  * conv_cal_mol_cm3   / sedcor_A_cm2
	new_sed(ised_13C_arg)    = sedcorfnp(p,ised_fnp_13C_arg)  * conv_arg_mol_cm3   / sedcor_A_cm2
!           calculate new sediment (planktonic proxy) stable isotopes
!           NOTE: foraminiferal calcite d18O is assumed to simply follow the oceanic d18O signal
	  new_sed(ised_18O_calfp) = new_sed(ised_cal) * &
	    & calc_frac_18O(calc_d18O(tracer(c,1,it_O2),tracer(c,1,it_18O_O2)))
	SELECT CASE (bioopt(bopt_d13CaCO3_fp))
	CASE ("1")
!           scheme #1 : after Mook et al. [1986]
!           NOTE: the Mook [1986] fractionation is approximated by assuming that 
!                 the d13C of HCO3- is the same as that of total DIC 
!                 (this is in order to be consistent with the calculation of benthic fractionation where 
!                 the fractionation within the aqueous carbonate system is not explicitly calculated)
	  new_sed(ised_13C_calfp) = new_sed(ised_cal) * calc_frac_13C( &
	    & calc_d13C(tracer(c,1,it_CO2),tracer(c,1,it_13C_CO2)) + &
	    & 15.10 - 4232.0 / tracer(c,1,it_T))
	CASE ("2")
!           scheme #2 : after Spero et al. [1997]
!           NOTE: G. bulloides relationship is taken to be the average of the 12th and 13th chamber 
!                 regressions resented by Spero et al. [1997] under constant total DIC conditions
	  new_sed(ised_13C_calfp) = new_sed(ised_cal) * calc_frac_13C( &
	    & calc_d13C(tracer(c,1,it_CO2),tracer(c,1,it_13C_CO2)) + &
	    & -0.135 - 13000.0 * sscc(c,isscc_cCO3))
	CASE ("3")
!           scheme #3 : after Spero et al. [1997]
!           NOTE: O. universa relationship is taken to be the average of the dark and high-light 
!                 regressions resented by Spero et al. [1997] under constant total DIC conditions
	  new_sed(ised_13C_calfp) = new_sed(ised_cal) * calc_frac_13C( &
	    & calc_d13C(tracer(c,1,it_CO2),tracer(c,1,it_13C_CO2)) + &
	    & +1.795 -  6000.0 * sscc(c,isscc_cCO3))
	CASE default
	  PRINT*,"FATAL ERROR [sue_sed.f90, couple_sedcor]:"
	  PRINT*,"option 'bioopt(bopt_d13CaCO3_fp)' is out of range;"
	  PRINT*,"selected number = ",bioopt(bopt_d13CaCO3_fp)
	  STOP
	END SELECT
!           calculate new sediment (benthic proxy) stable isotopes
!           NOTE: foraminiferal calcite d18O is assumed to simply follow the oceanic d18O signal
	new_sed(ised_18O_calfb) = new_sed(ised_cal) * &
	  & calc_frac_18O(calc_d18O(sedcortracer(p,it_O2),sedcortracer(p,it_18O_O2)))
	SELECT CASE (bioopt(bopt_d13CaCO3_fp))
	CASE ("1")
!           scheme #1 : after Mook et al. [1986]
!           NOTE: the Mook [1986] fractionation is approximated by assuming that 
!                 the d13C of HCO3- is the same as that of total DIC 
!                 (this is in order to be consistent with the calculation of benthic fractionation where 
!                 the fractionation within the aqueous carbonate system is not explicitly calculated)
	  new_sed(ised_13C_calfb) = new_sed(ised_cal) * calc_frac_13C( &
	    & calc_d13C(sedcortracer(p,it_CO2),sedcortracer(p,it_13C_CO2)) + &
	    & 15.10 - 4232.0 / sedcorpar(p,ised_par_T))
	CASE ("2")
!           scheme #2 : after Spero et al. [1997]
!           NOTE: G. bulloides relationship is taken to be the average of the 12th and 13th chamber 
!                 regressions resented by Spero et al. [1997] under constant total DIC conditions
	  new_sed(ised_13C_calfb) = new_sed(ised_cal) * calc_frac_13C(&
	    & calc_d13C(sedcortracer(p,it_CO2),sedcortracer(p,it_13C_CO2)) + &
	    & -0.135 - 13000.0 * sedcorpar(p,ised_par_CO3))
	CASE ("3")
!           scheme #3 : after Spero et al. [1997]
!           NOTE: O. universa relationship is taken to be the average of the dark and high-light 
!                 regressions resented by Spero et al. [1997] under constant total DIC conditions
	  new_sed(ised_13C_calfb) = new_sed(ised_cal) * calc_frac_13C(&
	    & calc_d13C(sedcortracer(p,it_CO2),sedcortracer(p,it_13C_CO2)) + &
	    & +1.795 -  6000.0 * sedcorpar(p,ised_par_CO3))
	CASE default
	  PRINT*,"FATAL ERROR [sue_sed.f90, couple_sedcor]:"
	  PRINT*,"option 'bioopt(bopt_d13CaCO3_fp)' is out of range;"
	  PRINT*,"selected number = ",bioopt(bopt_d13CaCO3_fp)
	  STOP
	END SELECT
!           calculate new sediment radioisotopes
!           NOTE: account for the initial 14C age of sedimenting material,
!                 in addition to the inital 14C/12C ratio
	new_sed(ised_14C_cal) = const_fracdecay_14C_ip * standard_14C_12C * new_sed(ised_cal)
!           calculate new sediment ages
	new_sed(ised_age_cal) = (R_tice + (1000.0 - R_tloop) / 1000.0) * new_sed(ised_cal)
!           determine whether the ash tracer should be added
!           NOTE: it is assumed that the ash occupies a negliable volume in the sediments
	IF ((I_tice == (t_ice_init - 1)) .AND. (I_tloop == 1)) THEN
	  new_sed(ised_ash)   = 1.0
	ELSE
	  new_sed(ised_ash)   = 0.0
	ENDIF
!           calculate volume of added material (as SOILD matter. i.e., zero porosity), in units of cm3 (cm-2)
!           NOTE: Porg, Norg, and Feorg, have only a 'virtual volume' and so are not counted
!                 this goes for the isotope and pH components as well
	new_sed_vol =  new_sed(ised_cal) + new_sed(ised_arg) + new_sed(ised_opal) + &
	  & new_sed(ised_refrac) + new_sed(ised_FeO)
!           check that the thickness of new sedimenting material is >= 0.0 cm yr-1
	IF (new_sed_vol < 0.0) THEN
	  PRINT*,"FATAL ERROR [sue_sedcor.f90, couple_sedcor]:"
	  PRINT*,"accumulating sediment < 0.0 cm yr-1"
	  PRINT*,"p,new_sed_th : ",p,new_sed_vol
	  PRINT*,"new_sed(ised_cal)     = ",new_sed(ised_cal)
	  PRINT*,"new_sed(ised_arg)     = ",new_sed(ised_arg)
	  PRINT*,"new_sed(ised_opal)    = ",new_sed(ised_opal)
	  PRINT*,"new_sed(ised_refrac)  = ",new_sed(ised_refrac)
	  PRINT*,"new_sed(ised_FeO)     = ",new_sed(ised_FeO)
	  PRINT*,"new_sed(ised_Corg)    = ",new_sed(ised_Corg)
	  STOP
	ENDIF
!           check that the thickness of new sedimenting material is < 1.0 cm yr-1
!           NOTE: this limit is imposed because of the way in which excess sedimentary material 
!                 is removed from the top layer and added to the sediment stack, 
!                 which has layers of thickness 1.0 cm
!           NOTE: take into account the porosity of the sediment stack layers
	IF ((new_sed_vol / (1.0 - par_sedcor_poros)) >= 1.0) THEN
	  PRINT*,"accumulating sediment thickness >= 1.0 cm yr-1"
	  PRINT*,"p,new_sed_th : ",p,(new_sed_vol / (1.0 - par_sedcor_poros))
	  PRINT*,"new_sed(ised_cal) th     = ",new_sed(ised_cal)    / (1.0 - par_sedcor_poros)
	  PRINT*,"new_sed(ised_arg) th     = ",new_sed(ised_arg)    / (1.0 - par_sedcor_poros)
	  PRINT*,"new_sed(ised_opal) th    = ",new_sed(ised_opal)   / (1.0 - par_sedcor_poros)
	  PRINT*,"new_sed(ised_refrac) th  = ",new_sed(ised_refrac) / (1.0 - par_sedcor_poros)
	  PRINT*,"new_sed(ised_FeO) th     = ",new_sed(ised_FeO)    / (1.0 - par_sedcor_poros)
	  PRINT*,"new_sed(ised_Corg) th    = ",new_sed(ised_Corg)   / (1.0 - par_sedcor_poros)
	  STOP
	ENDIF
      ELSE
!           use a fixed sedimentation rate
!           NOTE: fixed sedimentation rates are in units of (g cm-2 kyr-1)
!           NOTE: if the model is still in spin-up (i.e., the time is greater than the ininital model time),
!                 then use a second (alternative) sedimentation rate than is used later
!           NOTE: sedimentation units must be changed from (cm kyr-1) to (g cm-2 kyr-1)
	IF ((R_tice + (1000.0 - R_tloop) / 1000.0) > REAL(t_ice_init,KIND=fp_1)) THEN
	  new_sed(ised_Corg)       = sedcorfnp(p,ised_fnp_Corg) * conv_Corg_mol_cm3  / sedcor_A_cm2
	  new_sed(ised_FeO)        = 0.0
	  new_sed(ised_cal)        = sedcorpar(p,ised_par_sedcor_f2_cal) * conv_cal_g_cm3
	  new_sed(ised_arg)        = 0.0
	  new_sed(ised_opal)       = 0.0
	  new_sed(ised_refrac)     = sedcorpar(p,ised_par_sedcor_f2_refrac) * conv_refrac_g_cm3
	ELSE
	  new_sed(ised_Corg)       = sedcorfnp(p,ised_fnp_Corg) * conv_Corg_mol_cm3  / sedcor_A_cm2
	  new_sed(ised_FeO)        = 0.0
	  new_sed(ised_cal)        = sedcorpar(p,ised_par_sedcor_f1_cal) * conv_cal_g_cm3
	  new_sed(ised_arg)        = 0.0
	  new_sed(ised_opal)       = 0.0
	  new_sed(ised_refrac)     = sedcorpar(p,ised_par_sedcor_f1_refrac) * conv_refrac_g_cm3
	ENDIF
!           calculate new sediment (bulk material) stable isotopes
	new_sed(ised_13C_Corg)  = 0.0
	new_sed(ised_13C_cal)   = 0.0
	new_sed(ised_13C_arg)   = 0.0
!           calculate new sediment (proxy material) stable isotopes
	new_sed(ised_13C_calfp) = 0.0
	new_sed(ised_13C_calfb) = 0.0
	new_sed(ised_18O_calfp) = 0.0
	new_sed(ised_18O_calfb) = 0.0
!           calculate new sediment radioisotopes
!           NOTE: account for the initial 14C age of sedimenting material,
!                 in addition to the inital 14C/12C ratio
	new_sed(ised_14C_cal)   = const_fracdecay_14C_ip * standard_14C_12C * new_sed(ised_cal)
!           calculate new sediment ages
	new_sed(ised_age_cal)   = (R_tice + (1000.0 - R_tloop) / 1000.0) * new_sed(ised_cal)
!           determine whether the ash tracer should be added
!           NOTE: it is assumed that the ash occupies a negliable volume in the sediments
	IF ((I_tice == (t_ice_init - 1)) .AND. (I_tloop == 1)) THEN
	  new_sed(ised_ash)     = 1.0
	ELSE
	  new_sed(ised_ash)      = 0.0
	ENDIF
      ENDIF

!print*,"(c)"
! ***** (c) estimate dissolution/remineralization from sediment top ('well mixed') sedimentary layer
!           NOTE: sedimentary material is represented as SOILD matter (i.e., zero porosity)
!           NOTE: return all PO4 to the ocean to avoid having to balance the global PO4 budget
!           NOTE: return all NO3- to the ocean to avoid having to balance the global alkalinity budget
!           NOTE: retain all Feorg in the sediments
!           NOTE: allow no dissolution of refractory or ash tracer material
!           NOTE: remove isotopic components in proportion to their isotopic fraction in the sediment,
!                 but add small constant value to the demoninator to ensure that no divide-by-zero can occur
!           NOTE: because a sedimentary layer may be connected to two (or more) different ocean layers,
!                 the weighted average calcite and aragonite saturation depths must be used 
!                 in the estimation of interface calcite and aragonite dissolution
!           NOTE: when calculating the dissolution fluxes derived from top sediment Corg, cal, and arg 
!                 components, add a threshold factor to the denominator to ensure that 
!                 'divide-by-zero' problems are avoided
!           calculate Corg remineralization
!           NOTE: the Corg fluxes are left in units of (mol cm-2 yr)
!           NOTE: remineralize all Corg and 13Corg,
!                 so that the Corg and 13Corg budgets over long timescales balance 
!                 without the need for additional inputs to the ocean/atmosphere from external sources
!           NOTE: 'create' an extra fraction of Corg and 13Corg and add it to the new sedimentary material,
!                 so that changes in the Corg rain rate and in d13C can be kept track of
!           NOTE: do not bother to update the value of 'new_sed_vol', as it is not used again, 
!                 and its value will cannot go over 1.0 cm total from the creating of 0.1% new Corg volume
!           NOTE: the precise value of this fraction is unimportant, but should be small enough so that 
!                 Corg never attains a signficiant wt% of the sediment
!           NOTE: Corg remineralization must be calculated after the above check for sediment layers 
!                 being wholly or partly within the ocean mixed layer, as otherwise Corg tends 
!                 to accumulate in near-surface sediments, causing a gradual draw-down of C from the ocean
      dis_sed(ised_Corg)     = (1.0 - par_sed_presfrac_Corg) * new_sed(ised_Corg)
      dis_sed(ised_13C_Corg) = (1.0 - par_sed_presfrac_Corg) * new_sed(ised_13C_Corg)
!           calculate CaCO3 diagenesis
      IF (sedopt(sopt_sed_diagen_lookup_CaCO3) == yes) THEN
	CALL calc_sed_dis_CaCO3_lookup(dis_sed(:),new_sed(:),sedcortop(p,:),sedcorpar(p,:),sedcortracer(p,:))
      ELSE
	dis_sed(ised_cal)       = new_sed(ised_cal)
	dis_sed(ised_arg)       = new_sed(ised_arg)
	dis_sed(ised_13C_cal)   = new_sed(ised_13C_cal)
	dis_sed(ised_13C_calfp) = new_sed(ised_13C_calfp)
	dis_sed(ised_13C_calfb) = new_sed(ised_13C_calfb)
	dis_sed(ised_18O_calfp) = new_sed(ised_18O_calfp)
	dis_sed(ised_18O_calfb) = new_sed(ised_18O_calfb)
	dis_sed(ised_13C_arg)   = new_sed(ised_13C_arg)
      END IF
!           calculate opal diagenesis
      IF (sedopt(sopt_sed_diagen_lookup_opal) == yes) THEN
	CALL calc_sed_dis_opal_lookup(dis_sed(:),new_sed(:),sedcortop(p,:),sedcorpar(p,:),sedcortracer(p,:))
      ELSE
	dis_sed(ised_opal) = new_sed(ised_opal)
      ENDIF
!           calculate diagenesis of scavenged Fe and Fe in POM
        If (par_sed_FeO_fdis > (par_sed_presfrac_FeO * new_sed(ised_FeO))) then
           sed_FeO_fdis = par_sed_presfrac_FeO * new_sed(ised_FeO)
        else
           sed_FeO_fdis = par_sed_FeO_fdis
        endif
	dis_sed(ised_FeO) = (1.0 - par_sed_presfrac_FeO) * new_sed(ised_FeO) + sed_FeO_fdis
!           zero dissolution of refractory components
      dis_sed(ised_refrac)   = 0.0
      dis_sed(ised_ash)      = 0.0
!           check whether the sediment core lies within the mixed ocean layer
      IF (sedcorpar(p,ised_par_D_mid) <= phys(c,1,ip_D_bot)) THEN
	dis_sed(:) = 0.0 
      ENDIF
!           copy dissolution flux values to output array
      sedcorfdis(p,:) = dis_sed(:)
      
!print*,"(d)"
! ***** (d) update sediment stack
!           add the new sediment to the top sediment layer,  and deduct the calculated dissolved material
!           NOTE: all sediment quantities are in volume (cm3) of solid material
      sedcortop(p,:) = sedcortop(p,:) + new_sed(:) - dis_sed(:)
!           calculate temporary (i.e., beofre exchange with underlying sediments) top sediment composition
      top_sed_vol = sedcortop(p,ised_cal) + sedcortop(p,ised_arg) + sedcortop(p,ised_opal) + &
	& sedcortop(p,ised_refrac) + sedcortop(p,ised_FeO) + sedcortop(p,ised_Corg)
!           calculate the volume of material to be exchanged between the sediment stack and the top layer,
!           together with the equivalent sediment stack thicckness that the represents
!           NOTE: sedimentary material volume is as SOILD matter (i.e., zero porosity)
!           NOTE: use the ABSOLUTE value
!           NOTE: the thickness of sediment material to be exchanged between the top sediment and 
!                 sediment stack, is expressed using the sediment stack porosity
!           NOTE: the volume to be exchanged is calculated in this way, 
!                 using the difference between the updated (temporary) top sediment volume and the pre-
!                 determined volume (based on the defined thickness of the top layer and it's porosity), 
!                 rather than by the difference between new and dis volume, 
!                 to ensure that the top sediment thickness remains exactly as orignially set 
!                 (otherwise it tends to drift down very slightly)
      exe_sed_vol = ABS(top_sed_vol - ((1.0 - par_sedcor_poros_top) * par_sedcor_top_th))
      exe_sed_th  = ABS(top_sed_vol - ((1.0 - par_sedcor_poros_top) * par_sedcor_top_th)) / &
	& (1.0 - par_sedcor_poros)
!           calculate potential change in thickness of sediment top layer including new sedimenting material
!           NOTE: take into account the top layer porosity
!           NOTE: retain sign
      dsed_top_th = (top_sed_vol - ((1.0 - par_sedcor_poros_top) * par_sedcor_top_th)) &
	& / (1.0 - par_sedcor_poros_top)
!           calculate sub-layer number and thickness of top (incomplete) sub-layer of sediment stack
      n_sedcor_stack_top  = INT(sedcorpar(p,ised_par_top)) + 1
      sedcor_stack_top_th = sedcorpar(p,ised_par_top) - REAL((n_sedcor_stack_top - 1),KIND=fp_1)
!           keep thickness of top layer = par_sedcor_top_th by transfer to/from sediment stack,
!           (calculating a volume of sediment to be exchanged exchanged (exe_sed(:)):
!           - remove material to the sediment stack below if dsed_top_th >= 0.0 cm, or
!           - add material from the sediment stack below if dsed_top_th < 0.0 cm
!           NOTE: if dsed_top_th is zero then skip this section to potential 'avoid divide-by-zero' problems
      IF (dsed_top_th == 0.0) THEN
!          do nothing
	exe_sed(:) = 0.0
      ELSE
	IF (dsed_top_th > 0.0) THEN
!           add material to the sediment stack
!           test thickness of top (incomplete) sub-layer compared with thickness of material to be added:
!           - if exchange th <= remaining (unfilled) thickness of top sub-layer of sediment stack, then
!             add required material to top sub-layer only 
!           - if exchange th > remaining (unfilled) thickness of top sub-layer of sediment stack, then
!             add sufficient material to top sub-layer to fill it plus additional material to next layer up
!           NOTE: the porosity of the sediment stack must be taken into account
	  IF (exe_sed_th <= (1.0 - sedcor_stack_top_th)) THEN
!           calculate composition of material to be exchanged with sediment stack (as SOILD matter)
!           NOTE: the sediment material composition in 'sedtop' must be unit normalized,
!                 to give a compositional fraction
	    exe_sed(:) = exe_sed_vol * &
	      & (sedcortop(p,:) / par_sedcor_top_th) * (1.0 / (1.0 - par_sedcor_poros_top))
!           update sediment stack
	    sedcor(p,n_sedcor_stack_top,:) = sedcor(p,n_sedcor_stack_top,:) + exe_sed(:)
	  ELSE
!           calculate composition of material to be exchanged with sediment stack (as SOILD matter)
!           NOTE: the sediment material composition in 'sedtop' must be unit normalized,
!                 to give a compositional fraction
	    exe_sed(:) = exe_sed_vol * &
	      & (sedcortop(p,:) / par_sedcor_top_th) * (1.0 / (1.0 - par_sedcor_poros_top))
!           update sediment stack
!           NOTE: add material to the top (incomplete) sediment stack sub-layer in proportion to 
!                 the fraction of unfilled thickness over the equivalent thickness of total material to add
!                 add the remaining material to the next (completely unfilled) sub-layer up
	    sedcor(p,n_sedcor_stack_top,:) = sedcor(p,n_sedcor_stack_top,:) + &
	      & ((1.0 - sedcor_stack_top_th) / exe_sed_th) * exe_sed(:)
	    sedcor(p,(n_sedcor_stack_top + 1),:) = sedcor(p,(n_sedcor_stack_top + 1),:) + &
	      & (1.0 - ((1.0 - sedcor_stack_top_th) / exe_sed_th)) * exe_sed(:)
	  ENDIF
!           deduct exchange sediment material from the sediment top
	  sedcortop(p,:) = sedcortop(p,:) - exe_sed(:)
!           update sediment height and top sub-layer number
	  sedcorpar(p,ised_par_top) = sedcorpar(p,ised_par_top) + exe_sed_th
	  n_sedcor_stack_top = INT(sedcorpar(p,ised_par_top)) + 1
	ELSE
!           remove material from the sediment stack
!           test thickness of top (incomplete) sub-layer compared with thickness of material to be removed:
!           - if exchange vol <= thickness of top sub-layer of sediment stack, then
!             remove required material from the top stack sub-layer only 
!           - if exchange vol > thickness of top sub-layer of sediment stack, then
!             remove all material in the top stack sub-layer, plus additional material from next layer down
!           NOTE: the porosity of the sediment stack must be taken into account
	  IF (exe_sed_th <= sedcor_stack_top_th) THEN
!           calculate composition of material to be exchanged with sediment stack (as SOILD matter)
!           NOTE: the composition of the sediment in the top stack sub-layer must be unit normalized,
	    exe_sed(:) = exe_sed_vol * &
	      & (sedcor(p,n_sedcor_stack_top,:) / sedcor_stack_top_th) * (1.0 / (1.0 - par_sedcor_poros))
!           update sediment stack
	    sedcor(p,n_sedcor_stack_top,:) = sedcor(p,n_sedcor_stack_top,:) - exe_sed(:)
	  ELSE
!           calculate composition of material to be exchanged with sediment stack (as SOILD matter),
!           equal to ALL the material in the top (incomplete) sediment stack sub-layer,
!           plus a proportion of the material in the sub-layer immediately below
!           update sediment stack at the same time
	    exe_sed(:) = ((exe_sed_th - sedcor_stack_top_th) / exe_sed_th) * exe_sed_vol * &
	      & sedcor(p,(n_sedcor_stack_top - 1),:) * (1.0 / (1.0 - par_sedcor_poros))
	    sedcor(p,(n_sedcor_stack_top - 1),:) = sedcor(p,(n_sedcor_stack_top - 1),:) - exe_sed(:)
	    exe_sed(:) = exe_sed(:) + sedcor(p,n_sedcor_stack_top,:)
	    sedcor(p,n_sedcor_stack_top,:) = 0.0
	  ENDIF
!           add exchange sediment material to the sediment top
	  sedcortop(p,:) = sedcortop(p,:) + exe_sed(:)
!           update sediment height and top sub-layer number
	  sedcorpar(p,ised_par_top) = sedcorpar(p,ised_par_top) - exe_sed_th
	  n_sedcor_stack_top = INT(sedcorpar(p,ised_par_top)) + 1
	ENDIF
      ENDIF

!print*,"(e)"
! ***** (e) mix the sediment stack
      IF (sedopt(sopt_sedcor_mix) == yes) THEN
	CALL sed_mix(n_sedcor_com_min, &
	  & sedcor(p,(n_sedcor_stack_top - n_sed_mix):n_sedcor_stack_top,1:n_sedcor_com_min), &
	  & sedcortop(p,1:n_sedcor_com_min), &
	  & sedcorpar(p,:),sedmixk(:), &
	  & par_sedcor_top_th,sedcor_stack_top_th,par_sedcor_poros,par_sedcor_poros_top)
      ENDIF

!print*,"(f)"
! ***** (f) check the thickness of sediment stack
!           NOTE: if the sediment stack has reached the last available sub-layer, 
!                 then remove a number of sub-layer equal to 'n_sedcor_tot_init' from the bottom, 
!                 and shift the remaining sublayer down
      IF (n_sedcor_stack_top == n_sedcor_tot) THEN
!           shift sediment down the stack
	sedcor(p,1:n_sedcor_tot - n_sedcor_tot_init,:) = sedcor(p,(n_sedcor_tot_init + 1):n_sedcor_tot,:)
	sedcor(p,(n_sedcor_tot - n_sedcor_tot_init + 1):n_sedcor_tot,:) = 0.0
!           update sediment height and top sub-layer number
	sedcorpar(p,ised_par_top) = sedcorpar(p,ised_par_top) - REAL(n_sedcor_tot_init,KIND=fp_1)
	n_sedcor_stack_top = INT(sedcorpar(p,ised_par_top)) + 1
      ENDIF
      
    END DO! (end of sediment core ('p') loop)

  END SUBROUTINE couple_sedcor
! ****************************************************************************


! *************************************
! *** UPDATE SEDIMENT 14C INVENTORY ***
! *************************************
  SUBROUTINE update_sedcor_14C(sedcor,sedcortop)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sedcor_tot,n_sed_com_par)::sedcor
    REAL(KIND=fp_1),INTENT(inout),DIMENSION(n_sedcor,n_sed_com_par)::sedcortop
    
! *** decay 14C within sediments ***
! NOTE: it is easier to decay 14C in ALL sedimentary sub-layers, whether they are filled or not,
!       rather than having to calculate the sediment layer top
    sedcortop(:,ised_14C_cal) = const_fracdecay_14C * sedcortop(:,ised_14C_cal)
    sedcor(:,:,ised_14C_cal)  = const_fracdecay_14C * sedcor(:,:,ised_14C_cal)

  END SUBROUTINE update_sedcor_14C
! *************************************


! ******************************************************
! *** SAVE USER_DEFINABLE SEDIMENT CORE DATA TO FILE ***
! ******************************************************
  SUBROUTINE save_sedcor_data(sedcor,sedcortop,sedcorpar)
    IMPLICIT NONE
! dummy arguments
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sedcor_tot,n_sed_com_par)::sedcor
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sed_com_par)::sedcortop
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
! local variables
    INTEGER::p,o,oo
    INTEGER::out = 1
    INTEGER::n_sedcor_lim                                    !
    CHARACTER(LEN=255)::filename                             !
    REAL(KIND=fp_1)::sedcor_age_lim                          !
    REAL(KIND=fp_1)::sedcor_tot_wt                           ! total mass of solid coponents
    REAL(KIND=fp_1)::sedcor_tot_vol                          ! total volume of solid coponents
    INTEGER,DIMENSION(n_sedcor)::n_sedcor_stack_top          ! sediment stack top layer number
    REAL(KIND=fp_1),DIMENSION(n_sedcor)::sedcor_stack_top_th ! sediment stack top layer thickness
    REAL(KIND=fp_1),DIMENSION(n_sed_com_par)::sedcor_lim     ! 
    INTEGER::ash_max_o                ! running ash volume maximum sub-layer number
    REAL(KIND=fp_1)::ash_max          ! running ash volume maximum
    REAL(KIND=fp_1)::ash_max_depth    ! running ash volume maximum down-core depth
    REAL(KIND=fp_1)::ash_conv_dbs_age ! convert depth to age using ash stratigraphy

! *** initialize variables ***
! allocate array for holding sediment data reordered for writing to file
! NOTE: the array bounds extend from ZERO up to 'n_sedcor_tot' 
!       so that core top layer data can be more easily assimilated
    ALLOCATE(sedcor_save(n_sedcor,0:n_sedcor_tot,n_sed_com_par),STAT=alloc_error)  
    ALLOCATE(sedcor_save_age_tot(n_sedcor,0:n_sedcor_tot),STAT=alloc_error)     
    ALLOCATE(sedcor_save_age_ash(n_sedcor,0:n_sedcor_tot),STAT=alloc_error)     
    ALLOCATE(sedcor_save_age_14C(n_sedcor,0:n_sedcor_tot),STAT=alloc_error)   
    IF (error /= 0) THEN
! space for the sediment arrays could not be allocated
      PRINT*,"FATAL ERROR [sue_sedcor.f90, save_sedcor_data]:"
      PRINT*,"Space for the sediment arrays could not be allocated"
      STOP
    ENDIF
! zero local variables
    sedcor_save(:,:,:)       = 0.0
    sedcor_save_age_tot(:,:) = 0.0
    sedcor_save_age_ash(:,:) = 0.0

! *** transform sediment array for saving to file ***
! NOTE: the sediment array needs to be re-ordered so that the youngest sediment in the sediment stack 
!       starts with an array index of "1",
!       and the sediment top material is added at index position "0"
! NOTE: sediment composition descriptors ired to %calcite, such as age and pH,
!       need to be normailzed to %calcite
! NOTE: the sediment composition descriptors in the top layer sediments 
!       need to be normailzed to a thickness of 1.0 cm
! NOTE: the sediment composition descriptors in the top (incomplete) sub-layer of the sediment stack 
!       need to be normailzed to a thickness of 1.0 cm
! NOTE: the overall scheme is to loop through each sediment layer, and
!       (a) calculate local constants
!       (b) copy sediment core top layer data to data-file export array
!       (c) copy sediment core stack sub-layer data to data-file export array
!       (d) normailze age and pH values to the layer calcite content
!       (e) normalize ash (volume) content to unit (cm) layer thickness
!       (f) adjust core base data set values to improve MATLAB graphability
!       (g) normailze solid sediment components to a wt fraction (rather than solid volume) basis if required

! loop start
    DO p = 1,n_sedcor

! ***** (a) calculate local constants
      n_sedcor_stack_top(p)  = INT(sedcorpar(p,ised_par_top)) + 1
      sedcor_stack_top_th(p) = sedcorpar(p,ised_par_top) - REAL((n_sedcor_stack_top(p) - 1),KIND=fp_1)

! ***** (b) copy core top layer data
      sedcor_save(p,0,:) = sedcortop(p,:)

! ***** (c) copy core stack sub-layer data
      DO o = n_sedcor_stack_top(p),1,-1
	sedcor_save(p,(n_sedcor_stack_top(p) - o + 1),:) = sedcor(p,o,:)
      END DO

! ***** (d) adjust core base data set values - set unfilled sediment sublayers to 100% refractory
      DO o = 0,n_sedcor_tot
	IF (sedcor_save(p,o,ised_refrac) == 0.0) THEN
	  sedcor_save(p,o,ised_refrac) = 1.0
	ENDIF
      END DO

! ***** (g) assign a radiocarbon age
!           NOTE: ensure that age is in (kyr)
      DO o = 0,n_sedcor_tot
	IF ((sedcor_save(p,o,ised_14C_cal) > 0.0) .AND. (sedcor_save(p,o,ised_cal) > 0.0)) THEN
	  sedcor_save_age_14C(p,o) = -const_lamda_14C * &
	    & LOG(sedcor_save(p,o,ised_14C_cal) / (standard_14C_12C * sedcor_save(p,o,ised_cal)))
	  sedcor_save_age_14C(p,o) = conv_yr_kyr * sedcor_save_age_14C(p,o)
	ELSE
	  sedcor_save_age_14C(p,o) = 0.0
	ENDIF
      ENDDO

! ***** (g) normailze individual sediment component age values
      DO o = 0,n_sedcor_tot
	IF (sedcor_save(p,o,ised_cal) > 0.0) THEN
	  sedcor_save(p,o,ised_age_cal) = sedcor_save(p,o,ised_age_cal) / sedcor_save(p,o,ised_cal)
	ELSE
	  sedcor_save(p,o,ised_age_cal) = 0.0
	ENDIF
      ENDDO

! ***** (h) normalize ash (volume) content to unit (cm) layer thickness
      sedcor_save(p,0,ised_ash) = sedcor_save(p,0,ised_ash) / par_sedcor_top_th
      IF (sedcor_stack_top_th(p) > 0.0) THEN
	sedcor_save(p,1,ised_ash) = sedcor_save(p,1,ised_ash) / sedcor_stack_top_th(p)
      ENDIF
      DO o = 2,n_sed_tot
	sedcor_save(p,o,ised_ash) = sedcor_save(p,o,ised_ash) / 1.0
      END DO
      
! ***** (i) adjust core base data set values - adjust ages to be always monotonically increasing
      DO o = 1,n_sedcor_tot
	IF (sedcor_save(p,o,ised_age_cal) == 0.0) THEN
	  sedcor_age_lim = sedcor_save(p,o - 1,ised_age_cal)
	  n_sedcor_lim = o - 1
	  DO oo = n_sedcor_lim,n_sedcor_tot
	    sedcor_save(p,oo,ised_age_cal) = sedcor_age_lim + 0.001 * REAL((oo - n_sedcor_lim),KIND=fp_1)
	  END DO
	ENDIF
      END DO
      DO o = 1,n_sedcor_tot
	IF (sedcor_save_age_14C(p,o) == 0.0) THEN
	  sedcor_age_lim = sedcor_save_age_14C(p,o - 1)
	  n_sedcor_lim = o - 1
	  DO oo = n_sedcor_lim,n_sedcor_tot
	    sedcor_save_age_14C(p,oo) = sedcor_age_lim + 0.001 * REAL((oo - n_sedcor_lim),KIND=fp_1)
	  END DO
	ENDIF
      END DO

! ***** (g) normailze solid sediment components to a wt% basis (if required)
!           NOTE: as a first step, calculate total mass of solid components in the sediment sub-layer
!           NOTE: ensure that the stable isotopes are treated in the same manner
      IF (sedopt(sopt_sed_save_wtper) == yes) THEN
	DO o = 0,n_sedcor_tot
	  sedcor_tot_wt = conv_cal_cm3_g    * sedcor_save(p,o,ised_cal)    + &
	    &             conv_arg_cm3_g    * sedcor_save(p,o,ised_arg)    + &
	    &             conv_opal_cm3_g   * sedcor_save(p,o,ised_opal)   + &
	    &             conv_refrac_cm3_g * sedcor_save(p,o,ised_refrac) + &
	    &             conv_FeO_cm3_g    * sedcor_save(p,o,ised_FeO)    + &
	    &             conv_Corg_cm3_g   * sedcor_save(p,o,ised_Corg)
	  IF (sedcor_tot_wt > 0.0) THEN
	    sedcor_save(p,o,ised_cal)       = conv_cal_cm3_g    * sedcor_save(p,o,ised_cal)    / sedcor_tot_wt
	    sedcor_save(p,o,ised_arg)       = conv_arg_cm3_g    * sedcor_save(p,o,ised_arg)    / sedcor_tot_wt
	    sedcor_save(p,o,ised_opal)      = conv_opal_cm3_g   * sedcor_save(p,o,ised_opal)   / sedcor_tot_wt
	    sedcor_save(p,o,ised_refrac)    = conv_refrac_cm3_g * sedcor_save(p,o,ised_refrac) / sedcor_tot_wt
	    sedcor_save(p,o,ised_FeO)       = conv_FeO_cm3_g    * sedcor_save(p,o,ised_FeO)    / sedcor_tot_wt
	    sedcor_save(p,o,ised_Corg)      = conv_Corg_cm3_g   * sedcor_save(p,o,ised_Corg)   / sedcor_tot_wt
	    sedcor_save(p,o,ised_13C_Corg)  = conv_Corg_cm3_g * sedcor_save(p,o,ised_13C_Corg) / sedcor_tot_wt
	    sedcor_save(p,o,ised_13C_cal)   = conv_cal_cm3_g   * sedcor_save(p,o,ised_13C_cal) / sedcor_tot_wt
	    sedcor_save(p,o,ised_13C_arg)   = conv_arg_cm3_g   * sedcor_save(p,o,ised_13C_arg) / sedcor_tot_wt
	    sedcor_save(p,o,ised_14C_cal)   = conv_arg_cm3_g   * sedcor_save(p,o,ised_14C_cal) / sedcor_tot_wt
	    sedcor_save(p,o,ised_13C_calfp) = conv_cal_cm3_g * sedcor_save(p,o,ised_13C_calfp) / sedcor_tot_wt
	    sedcor_save(p,o,ised_13C_calfb) = conv_cal_cm3_g * sedcor_save(p,o,ised_13C_calfb) / sedcor_tot_wt
	    sedcor_save(p,o,ised_18O_calfp) = conv_cal_cm3_g * sedcor_save(p,o,ised_18O_calfp) / sedcor_tot_wt
	    sedcor_save(p,o,ised_18O_calfb) = conv_cal_cm3_g * sedcor_save(p,o,ised_18O_calfb) / sedcor_tot_wt
	  ENDIF
	END DO
      ELSE
	DO o = 0,n_sedcor_tot
	  sedcor_tot_vol = &
	    & sedcor_save(p,o,ised_cal)    + &
	    & sedcor_save(p,o,ised_arg)    + &
	    & sedcor_save(p,o,ised_opal)   + &
	    & sedcor_save(p,o,ised_refrac) + &
	    & sedcor_save(p,o,ised_FeO)    + &
	    & sedcor_save(p,o,ised_Corg)
	  IF (sedcor_tot_vol > 0.0) THEN
	    sedcor_save(p,o,ised_cal)       = sedcor_save(p,o,ised_cal)       / sedcor_tot_vol
	    sedcor_save(p,o,ised_arg)       = sedcor_save(p,o,ised_arg)       / sedcor_tot_vol
	    sedcor_save(p,o,ised_opal)      = sedcor_save(p,o,ised_opal)      / sedcor_tot_vol
	    sedcor_save(p,o,ised_refrac)    = sedcor_save(p,o,ised_refrac)    / sedcor_tot_vol
	    sedcor_save(p,o,ised_FeO)       = sedcor_save(p,o,ised_FeO)       / sedcor_tot_vol
	    sedcor_save(p,o,ised_Corg)      = sedcor_save(p,o,ised_Corg)      / sedcor_tot_vol
	    sedcor_save(p,o,ised_13C_Corg)  = sedcor_save(p,o,ised_13C_Corg)  / sedcor_tot_vol
	    sedcor_save(p,o,ised_13C_cal)   = sedcor_save(p,o,ised_13C_cal)   / sedcor_tot_vol
	    sedcor_save(p,o,ised_13C_arg)   = sedcor_save(p,o,ised_13C_arg)   / sedcor_tot_vol
	    sedcor_save(p,o,ised_14C_cal)   = sedcor_save(p,o,ised_14C_cal)   / sedcor_tot_vol
	    sedcor_save(p,o,ised_13C_calfp) = sedcor_save(p,o,ised_13C_calfp) / sedcor_tot_vol
	    sedcor_save(p,o,ised_13C_calfb) = sedcor_save(p,o,ised_13C_calfb) / sedcor_tot_vol
	    sedcor_save(p,o,ised_18O_calfp) = sedcor_save(p,o,ised_18O_calfp) / sedcor_tot_vol
	    sedcor_save(p,o,ised_18O_calfb) = sedcor_save(p,o,ised_18O_calfb) / sedcor_tot_vol
	  ENDIF
	END DO
      ENDIF

! ***** (h) produce stratigraphic marker age scale
!           NOTE: this assumes that the maximum ash volume fraction represents the ash impulse deposition age
!                 and that the sediment ages inbetween this depth and the surface
!                 can be linearly interpolated
!           NOTE: sediment deeper then the ash maximum is aged by linear extrapolation
!           NOTE: first, the ash maximum must be found
!           find ash maximum
      ash_max = 0.0
      DO o = 0,n_sedcor_tot
	IF (sedcor_save(p,o,ised_ash) > ash_max) THEN
	  ash_max   = sedcor_save(p,o,ised_ash)
	  ash_max_o = o
	ENDIF
      END DO
!           calculate ash maximum depth
      SELECT CASE (ash_max_o)
      CASE (0)
	ash_max_depth = par_sedcor_top_th / 2.0
      CASE (1)
	ash_max_depth = par_sedcor_top_th + sedcor_stack_top_th(p) / 2.0
      CASE default
	ash_max_depth = par_sedcor_top_th + sedcor_stack_top_th(p) + REAL((ash_max_o - 2),KIND=fp_1) + 0.5
      END SELECT
!           calculate linear age-depth relation
      ash_conv_dbs_age = REAL(t_ice_init,KIND=fp_1) / ash_max_depth
!           generate age scale
      o = 0
      sedcor_save_age_ash(p,o) = ash_conv_dbs_age * &
	& (par_sedcor_top_th / 2.0)
      o = 1
      sedcor_save_age_ash(p,o) = ash_conv_dbs_age * &
	& (par_sedcor_top_th + sedcor_stack_top_th(p) / 2.0)
      DO o = 2,n_sedcor_tot
	sedcor_save_age_ash(p,o) = ash_conv_dbs_age * &
	  & (par_sedcor_top_th + sedcor_stack_top_th(p) + REAL((o - 2),KIND=fp_1) + 0.5)
      END DO
!           cap age scale
      DO o = 0,n_sedcor_tot
	IF (sedcor_save_age_ash(p,o) >= REAL((t_spin_max + t_ice_init),KIND=fp_1)) THEN
	  sedcor_save_age_ash(p,o) = REAL((t_spin_max + t_ice_init),KIND=fp_1) + 0.001 * REAL(o,KIND=fp_1)
	ENDIF
      END DO

! 'p' loop end

    END DO

! *** save sediment tracer (z-axis) data ***
! NOTE: format for sediment data saving is:
!       (1) set filename
!       (2) open filepipe
!       (3) write ocean sediment depth interval mid-points as a file header
!       (4) save data in nested loop, first mixed layer then old sediment data
!       (5) close filepipe

! DATA: calcite (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sedcor"//"_cal.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.5)') &
	& (sedcor_save(p,o,ised_cal),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: aragonite (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sedcor"//"_arg.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.5)') &
	& (sedcor_save(p,o,ised_arg),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: CaCO3 (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sedcor"//"_CaCO3.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.5)') &
	& (sedcor_save(p,o,ised_cal) + sedcor_save(p,o,ised_arg),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: opal (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sedcor"//"_opal.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.5)') &
	& (sedcor_save(p,o,ised_opal),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: Corg (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sedcor"//"_Corg.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.5)') &
	& (sedcor_save(p,o,ised_Corg),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: refractory (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sedcor"//"_refrac.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.5)') &
	& (sedcor_save(p,o,ised_refrac),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: iron oxides (wt fraction or volume (cm3))
    filename = modelname//"_output/"//modelname//"_sedcor"//"_FeO.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.5)') &
	& (sedcor_save(p,o,ised_FeO),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: ash (volume (cm3))
    filename = modelname//"_output/"//modelname//"_sedcor"//"_ash.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.5)') &
	& (sedcor_save(p,o,ised_ash),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: d13Corg
    filename = modelname//"_output/"//modelname//"_sedcor"//"_d13C_Corg.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.3)') &
	& (calc_d13C(sedcor_save(p,o,ised_Corg),sedcor_save(p,o,ised_13C_Corg)),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: d13C (planktonic calcite)
    filename = modelname//"_output/"//modelname//"_sedcor"//"_d13C_cal.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.3)') &
	& (calc_d13C(sedcor_save(p,o,ised_cal),sedcor_save(p,o,ised_13C_cal)),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: d13C (planktonic aragonite)
    filename = modelname//"_output/"//modelname//"_sedcor"//"_d13C_arg.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.3)') &
	& (calc_d13C(sedcor_save(p,o,ised_arg),sedcor_save(p,o,ised_13C_arg)),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: d13C (planktonic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_sedcor"//"_d13C_calfp.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.3)') &
	& (calc_d13C(sedcor_save(p,o,ised_cal),sedcor_save(p,o,ised_13C_calfp)),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: d13C (benthic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_sedcor"//"_d13C_calfb.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.3)') &
	& (calc_d13C(sedcor_save(p,o,ised_cal),sedcor_save(p,o,ised_13C_calfb)),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: d18O (planktonic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_sedcor"//"_d18O_calfp.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.3)') &
	& (calc_d18O(sedcor_save(p,o,ised_cal),sedcor_save(p,o,ised_18O_calfp)),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! DATA: d18O (benthic foraminiferal calcite)
    filename = modelname//"_output/"//modelname//"_sedcor"//"_d18O_calfb.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.3)') &
	& (calc_d18O(sedcor_save(p,o,ised_cal),sedcor_save(p,o,ised_18O_calfb)),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)

! *** save sediment x-/y-axis data ***
! NOTE: format for sediment data saving is:
!       (1) set filename
!       (2) open filepipe
!       (3) write ocean sediment depth interval mid-points as a file header
!       (4) save data in nested loop, first mixed layer then old sediment data
!       (5) close filepipe

! X AXIS DATA: sediment age - calcite
    filename = modelname//"_output/"//modelname//"_sedcor"//"_age_cal.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.3)') &
	& (sedcor_save(p,o,ised_age_cal),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! X AXIS DATA: sediment age - 14C (planktonic calcite)
    filename = modelname//"_output/"//modelname//"_sedcor"//"_age_14C.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.3)') &
	& (sedcor_save_age_14C(p,o),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! X AXIS DATA: sediment age - ash
    filename = modelname//"_output/"//modelname//"_sedcor"//"_age_ash.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.3)') &
	& (sedcor_save_age_ash(p,o),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! X AXIS DATA: depth in sediment column
    filename = modelname//"_output/"//modelname//"_sedcor"//"_dbs.res"
    OPEN(unit=out,file=filename,action="write")
    o = 0
    WRITE(unit=out,FMT='(999F8.2)') &
      & ((par_sedcor_top_th / 2.0),p=1,n_sedcor)
    o = 1
    WRITE(unit=out,FMT='(999F8.2)') &
      & (par_sedcor_top_th + (sedcor_stack_top_th(p) / 2.0),p=1,n_sedcor)
    DO o = 2,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.2)') &
	& (par_sedcor_top_th + sedcor_stack_top_th(p) + REAL((o - 2),KIND=fp_1) + 0.5,p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! X AXIS DATA: sediment sub-layer thickness
    filename = modelname//"_output/"//modelname//"_sedcor"//"_layth.res"
    OPEN(unit=out,file=filename,action="write")
    o = 0
    WRITE(unit=out,FMT='(999F8.2)') &
      & (par_sedcor_top_th,p=1,n_sedcor)
    o = 1
    WRITE(unit=out,FMT='(999F8.2)') &
      & (sedcor_stack_top_th(p),p=1,n_sedcor)
    DO o = 2,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.2)') &
	& (REAL(1,KIND=fp_1),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)
! Y AXIS DATA: sediment layer depth in water column
    filename = modelname//"_output/"//modelname//"_sedcor"//"_depth.res"
    OPEN(unit=out,file=filename,action="write")
    DO o = 0,n_sedcor_tot
      WRITE(unit=out,FMT='(999F8.1)') &
	& (sedcorpar(p,ised_par_D_PIG_mid),p=1,n_sedcor)
    END DO
    CLOSE(unit=out)

! *** clean up ***
! deallocate local arrays
    DEALLOCATE(sedcor_save,STAT=dealloc_error)
    DEALLOCATE(sedcor_save_age_tot,STAT=alloc_error)     
    DEALLOCATE(sedcor_save_age_ash,STAT=alloc_error)     
    DEALLOCATE(sedcor_save_age_14C,STAT=alloc_error)   

! *** check for problems de-allocating array space ***
    IF (dealloc_error /= 0) THEN
      PRINT*,"FATAL ERROR [sue_sedcor.f90, save_sedcor_data]:"
      PRINT*,"Array space could not be de-allocated"
      STOP
    ENDIF

  END SUBROUTINE save_sedcor_data
! ******************************************************


! *********************************************************************
! *** SAVE USER_DEFINED SEDIMENT CORE WATER-SEDIMENT RAIN FLUX DATA ***
! *********************************************************************
  SUBROUTINE save_sedcor_fnp(tsave,sedcorfnp,sedcorpar,phys)
    IMPLICIT NONE
! dummy arguments
    INTEGER,INTENT(in)::tsave
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sed_fnp)::sedcorfnp
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::p,c
    INTEGER::out=1
    CHARACTER(LEN=255)::filename
    CHARACTER(LEN=3)::tsave_string

! *** create filename identifier string ***
    IF (tsave <= 9) THEN
      WRITE(tsave_string(3:3),'(i1)')tsave
      tsave_string(1:2) = "00"
    ELSE IF (tsave <= 99) THEN
      WRITE(tsave_string(2:3),'(i2)')tsave
      tsave_string(1:1) = "0"
    ELSE
      WRITE(tsave_string(1:3),'(i3)')tsave
    ENDIF

! *** save user-defined sediment core depth data ***
! NOTE: units : (m)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_depth.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      WRITE(unit=out,FMT='(999F10.3)') &
	& sedcorpar(p,ised_par_D_mid)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save calcite rain rate ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_cal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfnp(p,ised_fnp_cal) * conv_mol_umol) / (phys(c,1,ip_A_bot) * conv_m2_cm2)
    END DO
! close file pipe
    CLOSE(unit=out)
    
! *** save aragonite rain rate ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_arg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfnp(p,ised_fnp_arg) * conv_mol_umol) / (phys(c,1,ip_A_bot) * conv_m2_cm2)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save CaCO3 rain rate ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_CaCO3.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfnp(p,ised_fnp_cal) * conv_mol_umol) / (phys(c,1,ip_A_bot) * conv_m2_cm2) + &
	& (sedcorfnp(p,ised_fnp_arg) * conv_mol_umol) / (phys(c,1,ip_A_bot) * conv_m2_cm2)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save Corg rain rate ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_Corg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfnp(p,ised_fnp_Corg) * conv_mol_umol) / (phys(c,1,ip_A_bot) * conv_m2_cm2)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save opal rain rate ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_opal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfnp(p,ised_fnp_opal) * conv_mol_umol) / (phys(c,1,ip_A_bot) * conv_m2_cm2)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save Feorg rain rate ***
! NOTE: units : (nmol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_Feorg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfnp(p,ised_fnp_Feorg) * conv_mol_nmol) / (phys(c,1,ip_A_bot) * conv_m2_cm2)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save CaCO3/Corg rain ratio data ***
! NOTE: units : (dimensionless)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_CaCO3toCorg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="write")
! write data
    DO p = 1,n_sedcor
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfnp(p,ised_fnp_cal) + sedcorfnp(p,ised_fnp_arg)) / (0.000001 + sedcorfnp(p,ised_fnp_Corg))
    END DO
! close file pipe
    CLOSE(unit=out)
    
! *** save cal/Corg rain ratio data ***
! NOTE: units : (dimensionless)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_caltoCorg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="write")
! write data
    DO p = 1,n_sedcor
      WRITE(unit=out,FMT='(999F10.3)') &
	& sedcorfnp(p,ised_fnp_cal) / (0.000001 + sedcorfnp(p,ised_fnp_Corg))
    END DO
! close file pipe
    CLOSE(unit=out)
    
! *** save arg/Corg rain ratio data ***
! NOTE: units : (dimensionless)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_argtoCorg.res" 
! open file pipe
    OPEN(unit=out,file=filename,action="write")
! write data
    DO p = 1,n_sedcor
      WRITE(unit=out,FMT='(999F10.3)') &
	& sedcorfnp(p,ised_fnp_arg) / (0.000001 + sedcorfnp(p,ised_fnp_Corg))
    END DO
! close file pipe
    CLOSE(unit=out)
    
! *** save Corg remineralization data ***
! NOTE: in units of fraction of np remaining
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_remin_Corg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      WRITE(unit=out,FMT='(999F10.6)') (sedcorpar(p,ised_par_reminf_Corg))
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save opal remineralization data ***
! NOTE: in units of fraction of np remaining
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_remin_opal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      WRITE(unit=out,FMT='(999F10.6)') (sedcorpar(p,ised_par_reminf_opal))
    END DO
! close file pipe
    CLOSE(unit=out)
    
! *** save cal remineralization data ***
! NOTE: in units of fraction of np remaining
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_remin_cal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      WRITE(unit=out,FMT='(999F10.6)') (sedcorpar(p,ised_par_reminf_cal))
    END DO
! close file pipe
    CLOSE(unit=out)
    
! *** save arg remineralization data ***
! NOTE: in units of fraction of np remaining
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fnp_sedcor_remin_arg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      WRITE(unit=out,FMT='(999F10.6)') (sedcorpar(p,ised_par_reminf_arg))
    END DO
! close file pipe
    CLOSE(unit=out)

  END SUBROUTINE save_sedcor_fnp
! *********************************************************************


! ****************************************************************************
! *** SAVE USER_DEFINED SEDIMENT CORE WATER-SEDIMENT DISSOLUTION FLUX DATA ***
! ****************************************************************************
  SUBROUTINE save_sedcor_fdis(tsave,sedcorfdis,sedcorpar,phys)
    IMPLICIT NONE
! dummy arguments
    INTEGER,INTENT(in)::tsave
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sed_com_par)::sedcorfdis
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_sedcor,n_sed_par)::sedcorpar
    REAL(KIND=fp_1),INTENT(in),DIMENSION(n_col,n_lay,n_phys)::phys
! local variables
    INTEGER::p,c
    INTEGER::out=1
    CHARACTER(LEN=255)::filename
    CHARACTER(LEN=3)::tsave_string

! *** create filename identifier string ***
    IF (tsave <= 9) THEN
      WRITE(tsave_string(3:3),'(i1)')tsave
      tsave_string(1:2) = "00"
    ELSE IF (tsave <= 99) THEN
      WRITE(tsave_string(2:3),'(i2)')tsave
      tsave_string(1:1) = "0"
    ELSE
      WRITE(tsave_string(1:3),'(i3)')tsave
    ENDIF


! *** save user-defined sediment core depth data ***
! NOTE: units : (m)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fdis_sedcor_depth.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      WRITE(unit=out,FMT='(999F10.3)') &
	& sedcorpar(p,ised_par_D_mid)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save calcite dissolution flux ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fdis_sedcor_cal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfdis(p,ised_cal) * conv_cal_cm3_mol * conv_mol_umol)
    END DO
! close file pipe
    CLOSE(unit=out)
    
! *** save aragonite dissolution flux ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fdis_sedcor_arg.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfdis(p,ised_arg) * conv_arg_cm3_mol * conv_mol_umol)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save CaCO3 dissolution flux ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fdis_sedcor_CaCO3.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfdis(p,ised_cal) * conv_cal_cm3_mol * conv_mol_umol) + &
	& (sedcorfdis(p,ised_arg) * conv_arg_cm3_mol * conv_mol_umol)
    END DO
! close file pipe
    CLOSE(unit=out)

! *** save opal dissolution flux ***
! NOTE: units : (umol cm-2 yr-1)
    filename = modelname//"_output/"//modelname//"_"//tsave_string//"kaBP_t_ann_fdis_sedcor_opal.res"
! open file pipe
    OPEN(unit=out,file=filename,action="WRITE")
! write data
    DO p = 1,n_sedcor
      c = INT(sedcorpar(p,ised_par_colnum))
      WRITE(unit=out,FMT='(999F10.3)') &
	& (sedcorfdis(p,ised_opal) * conv_opal_cm3_mol * conv_mol_umol)
    END DO
! close file pipe
    CLOSE(unit=out)
    
  END SUBROUTINE save_sedcor_fdis
! *****************************************************************************


END MODULE sue_sedcor


